/* $Workfile: Vd5x16RawData.c $       $Revision: 9 $      $Modtime: 9/02/04 2:07p $  */
/*******************************************************************************

  Odin TeleSystems Inc.

  OTX Telecom Adapter Driver

  @doc PublicDemo

  @module

    Vd5x16RawData.c

    An example console application demonstrating the use of OTX Driver
    with a Thor-8-PCI boards and one 5x16 Vidar board connected over the CT Bus
    
    The sample application provides a simple console interface which utilizes
    simple keystrokes as commands.

    When two ore more boards are connected together with the appropriate board
    CT Bus interconnect ribbon cable it is possible to connect two devices with a pipe
    even if they are located on different board. (This example support multiple CT Buses
    with two Thor-8-PCI boards connected to each CT Bus interconnect cable.)

    Before pipes can connect over separate board the driver must be informed about
    what types of board interconnects exists, and how the boards are connected.
    This is done by opening the physical device OTX_DEVICE_CT_BUS (which is hosted by
    the driver device (0)). The physical device OTX_DEVICE_CT_BUS is a representation
    of the CT Bus cable which connects two or more boards. OtxDrvConnectBoard() is called
    to inform the OTX_DEVICE_CT_BUS device about which boards that is connected to it.

    The CT Bus does not have a direction like a pipe. The direction of each individual
    timeslot is determined at the time a Pipe Device must utilize the CT Bus
    to reach a different board. The direction of the pipe then determines the
    direction of that timeslot in the CT Bus.

    The CT Bus device has several highways (the actual leads of the cable). Each highway 
    is then multiplexed into several logical timeslots. One timeslot is always 64kbit/s 
    so the total number of timeslots per highway is then determined by the capacity of 
    each highway. The standard capacity is 2048 kbits/s which corresponds to 32 ts/hwy.


*******************************************************************************/
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <time.h>
#include "OsaStd.h"
#include "OsaCon.h"
#include "OsaThrd.h"
#include "OtxErr.h"
#include "OtxBrd.h"
#include "OtxDef.h"
#include "OtxType.h"
#include "OtxDrv.h"
#include "OtxTone.h"
#include "OtxDsp.h"
#include "OtxHdlc.h"
#include "OtxT1E1.h"
#include "OtxData.h"
#include "OtxRaw.h"

#define DEMO_TASK_REF 1L               // Task Reference for a non-blocking call
#define DEMO_DSP_MAX 8
#define DEMO_MAX_BOARDS 2
#define DEMO_DEFAULT_LOG_FILE "Vd5x16RawData.log"
#define DEMO_DEFAULT_NO_OF_T8P_BRDS 1

#define DEMO_LOGIC_DEVS_MAX 496   //Number of Channels for all the Li's
#define DEMO_RAW_TX_BUF_SIZE 2048 //initially 1024
#define DEMO_TEST_STRING "abcdefg"
#ifdef DEMO_LINK_WITH_LIBCMT
#include <process.h>
#endif

typedef struct DemoEnvT {
  OTX_BOOL        bExitOnError;
  char           *logFileName;
  OtxT1E1LiModeE  eT1E1Mode;// T1 or E1 mode
  OtxDataFormatE  law;
  OTX_INT32       coCtBuses;
  OTX_INT32       coBrds;
  OTX_INT32       coLi;
  OTX_INT32       nStartingTimeslot;
  OTX_INT32       nRawDataCapacity;
} DemoEnvT;


/*******************************************************************************
 Function Prototypes
*******************************************************************************/
static void DemoCreateEventQueue(void);
static void DemoOpenPhysicalDevices(void);
static void DemoOpenPhysicalDevicesVidar(void);
static void DemoConfigureE1T1Devices(void);
static void DemoEnablePhysicalDevices(void);
static void DemoEnablePhysicalDevicesVidar(void);
static void DemoCreateLogicalDevices(void);
static void DemoConnectDevices(void);
static void DemoEnableLogicalDevices(void);
static void DemoInitT1E1ConfigData(OUT OtxE1ConfigOptionsS *pE1CfgData, OUT OtxT1ConfigOptionsS *pT1CfgData);
static void DemoPrintEvent(IN OtxEventDataS *pEventData);
static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT otxRc);
static void DemoExitQuestion(void);
static void DemoLogf(char *formatStr, ...);
static void DemoHelpPanel(void);
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP);

/*******************************************************************************
 Global Data
*******************************************************************************/
OTX_EVENT   g_hNotificationEvent = OTX_INVALID_EVENT_VALUE;
OTX_HANDLE  g_hEventQueue  = 0;
FILE       *g_pDemoLogFile = NULL;
DemoEnvT    g_demoEnv;
OTX_HANDLE  g_hCtBus[16];
OTX_INT32   g_nMsgLen = 10;
OTX_BOOL    g_bHdlcVerbose = OTX_TRUE;
OTX_BOOL    g_bStatsThreadActive = OTX_FALSE;


struct {
  OTX_HANDLE hThorController;
  OTX_HANDLE hVidarAsmController;
  OTX_HANDLE hDsp[4];
  OTX_HANDLE hQuadE1T1[2]; 
  OTX_HANDLE hLi[8];
} g_hPhysDevs;

struct {
  OtxE1ConfigOptionsS e1CfgData;
  OtxT1ConfigOptionsS t1CfgData;
} g_T1E1CfgData;

struct sRawDataSender {
  OTX_HANDLE hHandle;
  OTX_HANDLE hEventQueue;
  OTX_EVENT  hNotificationEvent;
  OTX_UINT32 nSourceId;
  OTX_INT32   cbSent;
  OTX_BOOL    bActive;
  OTX_UINT8   sqTxBuf[DEMO_RAW_TX_BUF_SIZE];
  FILE       *pTxFile;
  OTX_INT32   cbTxBuf;
  OTX_INT32   coUnderRunErrors;
};

struct {
  OTX_HANDLE hVidarController;
  OTX_HANDLE hDsp[16];
} g_hPhysDevsVidar;

struct {
  OTX_HANDLE hDspLiRawData;
  OTX_HANDLE hAsmDspLiRawData;
  OTX_INT32 nDsp;
  OTX_INT32 nLi;
} g_hPipes[DEMO_LOGIC_DEVS_MAX];

struct {
  struct sRawDataSender mRawDataSender[8];
} g_hLogicDevs;


/*******************************************************************************

  @func

  Create the event queue to be used by the driver to inform the application
  about events, such as line status changes, or detected DMTF digits.

*******************************************************************************/
void DemoCreateEventQueue(void)
{
  OTX_RESULT result;
  
  // Create a notification Event
  result = OtxDrvCreateEvent(&g_hNotificationEvent);
  if (OTX_RESULT_CODE(result) == OTX_E_NOTIMPL) {
    g_hNotificationEvent = NULL;    // D16V86 apps cannot use event objects
    result = OTX_S_OK;
  }
  
  DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
  
  // Create Event Queue for driver notifications
  result = OtxDrvCreateEventQueue(128, g_hNotificationEvent, &g_hEventQueue); DemoCheckRc(g_hEventQueue, result);
  
  // Enable Event Queue
  result = OtxDrvEnable(g_hEventQueue, OTX_TASK_SYNC,0); DemoCheckRc(g_hEventQueue, result);
}



/*******************************************************************************

  @func
    Open the Physical Devices on the Thor-8-PCI board

  @comm
    Calls function <f OtxDrvCreatePhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.

*******************************************************************************/
void DemoOpenPhysicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;
  OTX_INT32  nQuadE1T1; // Only used for Thor-8-PCI
  
  result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_THOR_8_PCI, 0, 0, g_hEventQueue, &g_hPhysDevs.hThorController); DemoCheckRc(g_hPhysDevs.hThorController, result);
  
  // On the Thor-8-PCI board the E1T1 devices are hosted by a quad E1T1 device. Open those two devices.
  for (nQuadE1T1 = 0; nQuadE1T1 < 2; nQuadE1T1++) {
    DemoLogf("Open OTX_DEVICE_QUAD_T1E1 #%d\n", nQuadE1T1);
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hThorController, OTX_DEVICE_QUAD_T1E1, nQuadE1T1, nQuadE1T1, g_hEventQueue, &g_hPhysDevs.hQuadE1T1[nQuadE1T1]); DemoCheckRc(g_hPhysDevs.hThorController, result);
  }
  
  // Get handles to T1/E1 line interface devices 
  for (i = 0; i < 8; i++) {
    nQuadE1T1 = i/4;
    DemoLogf("Open OTX_DEVICE_LI_T1E1 #%d (hosted by OTX_DEVICE_QUAD_T1E1 #%d)\n", i, nQuadE1T1);
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevs.hQuadE1T1[nQuadE1T1], OTX_DEVICE_LI_T1E1, i%4, i*4, g_hEventQueue, &g_hPhysDevs.hLi[i]); DemoCheckRc(g_hPhysDevs.hQuadE1T1[nQuadE1T1], result);
  }
  DemoLogf("Successfully opened the lines\n");    
}

/*******************************************************************************

  @func
    Open the Physical Devices on the Vidar 5x16 board

  @comm
    Calls function <f OtxDrvCreatePhysicalDevice> returning a handle to a
    Device Object representing a physical device, which can be the
    entire board or an integrated circuit (chip) on the board. The number of
    physical devices is set per board and cannot change dynamically.

*******************************************************************************/

void DemoOpenPhysicalDevicesVidar(void)
{
  
  OTX_RESULT result;
  OTX_INT32  i;
  
  // Connect to the Physical Vidar-5x16 Board Controller
  DemoLogf("Open OTX_DEVICE_VIDAR_5x_PCI (Vidar-5x16-PCI board)\n");
  result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_VIDAR_5x_PCI, 0, 0, g_hEventQueue, &g_hPhysDevsVidar.hVidarController); DemoCheckRc(g_hPhysDevsVidar.hVidarController, result);
  
  for (i = 0; i < 8; i++) {
    result = OtxDrvOpenPhysicalDevice(g_hPhysDevsVidar.hVidarController, OTX_DEVICE_DSP, i, i, g_hEventQueue, &g_hPhysDevsVidar.hDsp[i]); DemoCheckRc(g_hPhysDevsVidar.hVidarController, result);
  }
  
  // Load DSP Program
  for (i = 0; i < 8; i++) {
    DemoLogf("Loading OTX_DSP_SPM_OTXHDLC to DSP#%d \n", i);
    result = OtxDspRunSpm(g_hPhysDevsVidar.hDsp[i], OTX_DSP_SPM_OTXHDLC); DemoCheckRc(g_hPhysDevsVidar.hDsp[i], result);
  }
  
}



/*******************************************************************************

  @func
    Configure E1/T1 Devices

  @comm The clock source of the board(s) must set by a call 
  to <f OtxBrdSetClocks> prior to calling this function.

*******************************************************************************/
void DemoConfigureE1T1Devices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;
  // Set default configuration data
  DemoInitT1E1ConfigData(&(g_T1E1CfgData.e1CfgData), &(g_T1E1CfgData.t1CfgData));
  
  if (g_demoEnv.eT1E1Mode == OTX_T1E1_MODE_T1){ // T1
    // Configure T1 Transceivers
    for (i = 0; i < 8; i++) {
      result = OtxT1SetConfiguration(g_hPhysDevs.hLi[i], &(g_T1E1CfgData.t1CfgData));  DemoCheckRc(g_hPhysDevs.hLi[i], result);
    }
    // Set them in T1 mode
    for (i = 0; i < 8; i++) {
      result =  OtxT1E1SetLiMode(g_hPhysDevs.hLi[i], OTX_T1E1_MODE_T1); DemoCheckRc(g_hPhysDevs.hLi[i], result);
    }
  }
  else{  // E1
    // Configure E1 Transceivers
    for (i = 0; i < 8; i++) {
      result = OtxE1SetConfiguration(g_hPhysDevs.hLi[i], &(g_T1E1CfgData.e1CfgData));  DemoCheckRc(g_hPhysDevs.hLi[i], result);
    }
    // Set them in E1 mode
    for (i = 0; i < 8; i++) {
      result =  OtxT1E1SetLiMode(g_hPhysDevs.hLi[i], OTX_T1E1_MODE_E1); DemoCheckRc(g_hPhysDevs.hLi[i], result);
    }
    
  }
}

/*******************************************************************************

  @func
    Enable Configured Thor-8-PCI Physical Devices

*******************************************************************************/
void DemoEnablePhysicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;

  result = OtxDrvEnable(g_hPhysDevs.hThorController,   OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hThorController, result);
  for (i = 0; i < 8; i++) {
    result = OtxDrvEnable(g_hPhysDevs.hLi[i], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevs.hLi[i], result);
  }
  
}

/*******************************************************************************

  @func
    Enable Configured Vidar 5x16 Physical Devices

*******************************************************************************/
void DemoEnablePhysicalDevicesVidar(void)
{
  OTX_RESULT result;
  OTX_INT32  i;

  result = OtxDrvEnable(g_hPhysDevsVidar.hVidarController, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevsVidar.hVidarController, result);
  for (i = 0; i < 8; i++) {
    result = OtxDrvEnable(g_hPhysDevsVidar.hDsp[i], OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hPhysDevsVidar.hDsp[i], result);
  }
}


/*******************************************************************************

  @func
    Create the needed Logical Devices

  @comm
    The logical devices are differenciated from the phyiscal devices by the fact
    the number of logical devices is not set. More of them can be requested
    by the user and allocated by the system if needed.

*******************************************************************************/
void DemoCreateLogicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32 i;
  OTX_UINT32 nIrpQueueLimit;
  
  
  //If there is only one Thor-8-PCI board with Vidar 5x16 , then 8Li's of the Thor board are connected to the 8DSP's on the Vidar 5x16 board
  for (i = 0; i < 8; i++) {
    // Create a notification Event and Event Queue for the Raw Data Sender
    result = OtxDrvCreateEvent(&g_hLogicDevs.mRawDataSender[i].hNotificationEvent); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvCreateEventQueue(128, g_hLogicDevs.mRawDataSender[i].hNotificationEvent, &g_hLogicDevs.mRawDataSender[i].hEventQueue); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hEventQueue, result);
    result = OtxDrvEnable(g_hLogicDevs.mRawDataSender[i].hEventQueue, OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hEventQueue, result);
    
    DemoLogf("Creating RawData Sender [%d] -> DSP# [%d] on Vidar 5x16 Board \n", i, i);
    // Create Raw Data Sender
    result = OtxDrvCreateLogicalDevice(g_hPhysDevsVidar.hDsp[i], OTX_LDEVICE_DATA_RAW_SENDER, g_hLogicDevs.mRawDataSender[i].hEventQueue, i, &g_hLogicDevs.mRawDataSender[i].hHandle); DemoCheckRc(g_hPhysDevsVidar.hDsp[i], result);
    g_hLogicDevs.mRawDataSender[i].nSourceId = i; // Store sourceId so that we can easily print it from the Sender Thread
    
    // Increase the IRP Queue limit for fast pipes (to be able to buffer more transmit data in the driver)
    result = OtxDrvGetAttributeValueUint32(g_hLogicDevs.mRawDataSender[i].hHandle, OTX_ATTR_DEVICE_IRP_QUEUE_LIMIT, &nIrpQueueLimit); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hHandle, result);
    //DemoLogf("RawDataSender[%d]: nIrpQueueLimit = %d\n", i, nIrpQueueLimit);
    if (g_demoEnv.nRawDataCapacity == 64) {
      nIrpQueueLimit = 8; //check if dsp's could handle better if the irp queue limit is reduced to 8
      DemoLogf("RawDataSender[%d]: Setting nIrpQueueLimit = %d\n", i, nIrpQueueLimit);
      result = OtxDrvSetAttributeValueUint32(g_hLogicDevs.mRawDataSender[i].hHandle, OTX_ATTR_DEVICE_IRP_QUEUE_LIMIT, nIrpQueueLimit); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hHandle, result);
    }
  }
  
}




/*******************************************************************************

  @func
    Create the needed Pipe Devices and connect the physical and logical devices
    to pipes.

  @comm
    A Pipe Device is a logical entity representing a serial bit-stream (a subset of a
    Highway) with a data rate between 8 kbit/s and 2.048Mbit/s. A pipe is a characterized
    by its capacity (in 8 kbit/s increments). The user can create a pipe with
    the driver function <f OtxDrvCreatePipe>. A pipe is connected to physical or
    logical devices and mapped to specific time-slots with the driver function
    <f OtxDrvConnectPipe>.

*******************************************************************************/
void DemoConnectDevices(
  void
  )
{
  OTX_INT32 nTs;
  OTX_INT32 j;
  OTX_RESULT result;
  OTX_UINT8 sqTimeSlotMask[32];
  
  // Create the Pipes (one for each direction) between the 8 DSPs on the Vidar 5x16 board and the 8 LIs of Thor-8-PCI board 0
  for (j = 0; j < 8; j++) {
    OTX_INT32 nCapacity;
    // Create Raw Data pipes (one for each direction) Use all timeslots except TS0.
    memset(sqTimeSlotMask, 0, 32 * sizeof(OTX_UINT8));
    // Standard (64kpbs) or superchannel
    nCapacity = 0;
    DemoLogf("Creating pipe for RawData Sender/Reciever starting at TS%d\n", 1);
    for (nTs = 0; nTs < 32; nTs++) {
      if (nCapacity == g_demoEnv.nRawDataCapacity) {
        break;
      }
      if (nTs == 0) {
        continue; // TS0 is sync timeslot
      }
      sqTimeSlotMask[nTs] = 0xFF; 
      nCapacity += 64;
    }
    if (nCapacity != g_demoEnv.nRawDataCapacity) {
      DemoLogf("Error: Could not connect Raw Data Sender/Receiver with capacity=%d\n", g_demoEnv.nRawDataCapacity);
      OsaConsoleExit();
      OtxDrvReleaseLib();
      exit(1);
    }
    DemoLogf("Creating pipe [%d] of Capacity of %dKbps\n", j, nCapacity);
    // Create Raw Data pipes (one for each direction) Use all timeslots except TS0.
    result = OtxDrvCreatePipe(nCapacity, &g_hPipes[j].hDspLiRawData); DemoCheckRc(g_hPipes[j].hDspLiRawData, result);
    DemoLogf("Successfully created RawData Sender pipes\n");    
    //Connect Raw Data Sender pipes to the DSP  as input device and the Li as the output device
    DemoLogf("Connecting Raw Data Sender pipe [%d] to Vidar 5x16 DSP# [%d] \n", j, j);
    result = OtxDrvConnectPipe(g_hPipes[j].hDspLiRawData, OTX_PIPE_INPUT,  g_hPhysDevsVidar.hDsp[j],  -1 , NULL, 0); DemoCheckRc(g_hPipes[j].hDspLiRawData, result);
    DemoLogf("Connected the INPUT for the RawData sender pipe [%d]\n", j);
    result = OtxDrvConnectPipe(g_hPipes[j].hDspLiRawData, OTX_PIPE_OUTPUT, g_hPhysDevs.hLi[j], -1, sqTimeSlotMask, 32); DemoCheckRc(g_hPipes[j].hDspLiRawData, result);
    DemoLogf("Connected the OUTPUT pipe for the RawData sender pipe [%d] at TimeSlot %d \n", j, nTs);
    
    // CONNECT LOGICAL DEVICES WITHIN THE DSP
    // Connect Raw Data Sender to the Raw Data Pipe
    DemoLogf("Connecting Logical Device : RawData Sender [%d] Handle to Pipe [%d] \n", j, j);    
    result = OtxDrvConnectLogicalDevice(g_hLogicDevs.mRawDataSender[j].hHandle, -1, g_hPipes[j].hDspLiRawData, -1); DemoCheckRc(g_hLogicDevs.mRawDataSender[j].hHandle, result);
    DemoLogf("Connected Logical Device : RawData Sender [%d] Handle to Pipe [%d] \n", j, j);    
    DemoLogf("-------------------------------------------------------------------------\n");
    
  }
}


/*******************************************************************************

  @func
    Initialize the logical device. This function must be called after
    the devices are created and connected.

*******************************************************************************/
void DemoInitializeLogicalDevices(
  void  
)
{
  OTX_RESULT result;
  OTX_INT32  i;
  
  for (i = 0; i < 8; i++) {
    DemoLogf("Initializing RawData Sender [%d] \n", i);
    // Initialize the Raw Data Senders (hosted in the DSP)
    result = OtxRawDataSenderInit(
      g_hLogicDevs.mRawDataSender[i].hHandle, 
      OTX_DATA_FORMAT_UINT8,  // No data conversion. 
      0xFF,        // Flag to be sent when the sender is out of data
      OTX_TRUE     // Least Significant part is transmitted first. Only applicable for a subchannel pipe (nCapacity < 64).
      ); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hHandle, result);
  }
}


/*******************************************************************************

  @func
    Enable Configured Logical Devices

*******************************************************************************/
void DemoEnableLogicalDevices(void)
{
  OTX_RESULT result;
  OTX_INT32  i;

  for (i = 0; i < 8; i++) {
    DemoLogf("Enable mRawDataSender[%d]\n", i);
    result = OtxDrvEnable(g_hLogicDevs.mRawDataSender[i].hHandle,  OTX_TASK_SYNC, 5000L); DemoCheckRc(g_hLogicDevs.mRawDataSender[i].hHandle, result);

  }
  
}


/*******************************************************************************

  @func
    Initialize the T1/E1 Configuration Data Structures passed to the T1/E1
    Line Interface physical devices

*******************************************************************************/
void DemoInitT1E1ConfigData(
  OUT OtxE1ConfigOptionsS *pE1CfgData,
  OUT OtxT1ConfigOptionsS *pT1CfgData
)
{
  // Initialize E1 Configuration Data
  pE1CfgData->eTxLineCode                = OTX_E1_LC_HDB3;
  pE1CfgData->eRxLineCode                = OTX_E1_LC_HDB3;
  pE1CfgData->eTxFrameFormat             = OTX_E1_FF_CRC4_MULTIFRAME;
  pE1CfgData->eRxFrameFormat             = OTX_E1_FF_CRC4_MULTIFRAME;
  pE1CfgData->eAisDetection              = OTX_E1_AIS_ETS300233;
  pE1CfgData->bfSiBits                   = 0x3;
  pE1CfgData->bfSaBits                   = 0x1F;
  pE1CfgData->bExtendedHDB3errorDetect   = OTX_FALSE;
  pE1CfgData->bAutoRegainMultiframe      = OTX_TRUE;
  pE1CfgData->bAutoResync                = OTX_TRUE;
  pE1CfgData->bAutoCrcBitInversion       = OTX_FALSE;
  pE1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
  pE1CfgData->nLosSensitivity            = 128;
  pE1CfgData->nLosRecovery               = 128;
  pE1CfgData->nLineLength                = 10;
  pE1CfgData->bHighTxPower               = OTX_FALSE;
  pE1CfgData->bReceiveEqualizer          = OTX_FALSE;

  // Initialize T1 Configuration Data
  pT1CfgData->eSignalingMode             = OTX_T1_SM_CCS;
  pT1CfgData->eTxLineCode                = OTX_T1_LC_B8ZS;
  pT1CfgData->eRxLineCode                = OTX_T1_LC_B8ZS;
  pT1CfgData->eFrameFormat               = OTX_T1_FF_ESF;
  pT1CfgData->bCRC6Enable                = OTX_TRUE;
  pT1CfgData->eTxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
  pT1CfgData->eRxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
  pT1CfgData->bAutoResync                = OTX_TRUE;
  pT1CfgData->bAutoCrcBitInversion       = OTX_FALSE;
  pT1CfgData->eLfaSensitivity            = OTX_T1_LFA_2_OUT_OF_4;
  pT1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
  pT1CfgData->nLosSensitivity            = 128;
  pT1CfgData->nLosRecovery               = 128;
  pT1CfgData->nLineLength                = 10;
  pT1CfgData->bHighTxPower               = OTX_FALSE;
  pT1CfgData->bReceiveEqualizer          = OTX_TRUE;
  pT1CfgData->bfClearChannels            = 0x000000; // 0x000000 = No channels are marked as clear channels
}


static void DemoCheckRc(IN OTX_HANDLE hDevice, IN OTX_RESULT result)
/*******************************************************************************
  @func
    Check the result from API call. If the result is a failure code then prompt
    the user if he wants to exit.
*******************************************************************************/
{
  if ( OTX_FAILURE(result) ) {

    if (hDevice == OTX_INVALID_HANDLE_VALUE) {
      DemoLogf("ERROR(%#lX) \n", result);
    } else {
      OTX_RESULT otxRc;
      char msgStr[256];
      OtxAttribTypeE  eAttribType;    // Type of the attribute, if attribute exists
      OTX_UINT32      cbAttribValue;  // Size in bytes of attribute value (Get value with <f OtxDrvGetAttributeValue>)
      OTX_BOOL        bReadOnly;      // Is attribute writable or read-only
      OTX_UINT32      eDeviceType = 999;    // Device type for hDevice 
      OTX_UINT32      nDeviceNumber = 999;  // Device number for hDevice
      
      // Get the Device Type for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_TYPE, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_TYPE, &eDeviceType); 
      } 
      
      // Get the Device Number for hHandle
      otxRc = OtxDrvGetAttributeType(hDevice, OTX_ATTR_DEVICE_NO, &eAttribType, &cbAttribValue, &bReadOnly); 
      if (OTX_SUCCESS(otxRc)) {
        assert(eAttribType == OTX_ATTRIB_UINT32);
        assert(cbAttribValue == sizeof(OTX_UINT32));
        OtxDrvGetAttributeValueUint32(hDevice, OTX_ATTR_DEVICE_NO, &nDeviceNumber); 
      }
      
      // Convert the result code to a more descriptive string
      otxRc = OtxDrvResultCode2String(hDevice, result, 256, &msgStr[0]);
      if (OTX_FAILURE(otxRc)) {
        strcpy(msgStr, "Cannot translate this result code.");
      }
      DemoLogf("ERROR(%#lX [%d.%d]) Dev(%X,%d,%d): %s \n", OTX_RESULT_CODE(result), OTX_RESULT_SOURCE_ID(result), OTX_RESULT_LINE_NUM(result), hDevice, (int) eDeviceType, (int)nDeviceNumber, msgStr);
    }

    DemoExitQuestion();
  }
}


/*******************************************************************************
  @func
    Prompts whether to exit the program. Usually called after an error 
    has been reported.

*******************************************************************************/
void DemoExitQuestion()
{
  short c;

  if (g_demoEnv.bExitOnError) {
    c = 'y';
  } else {
    DemoLogf("Do you want to exit (Y/N) ");
    c = OsaConsoleGetCh();
  }
  if (c == 'y' || c == 'Y') {
    // Clean up
    OtxDrvReleaseLib();
    OsaConsoleExit();
    exit(1);
  }
}


/*******************************************************************************

  @func
    Print the state of a device

  @comm

*******************************************************************************/
void DemoPrintDeviceState(
  OTX_HANDLE hHandle
)
{
  OTX_RESULT result;
  OtxDeviceStateE  eDevState;
  char *szDevState;

  result = OtxDrvGetState(hHandle, &eDevState); DemoCheckRc(hHandle, result);
  switch (eDevState) {
  case OTX_DEVICE_STATE_UNKNOWN:         
    szDevState = "OTX_DEVICE_STATE_UNKNOWN";
    break;
  case OTX_DEVICE_WORKING:
    szDevState = "OTX_DEVICE_WORKING";
    break;
  case OTX_DEVICE_BLOCKED_AUTO:
    szDevState = "OTX_DEVICE_BLOCKED_AUTO"; 
    break;
  case OTX_DEVICE_BLOCKED_MANUAL:
    szDevState = "OTX_DEVICE_BLOCKED_MANUAL";
    break;
  default:
    szDevState = "Invalid Device state";
    break;
  }
  DemoLogf("Device State(0x%X): %s\n", hHandle, szDevState);
}
 



/*******************************************************************************
  @func
    Idle Function that check for Events from the OTX Driver
*******************************************************************************/
void *DemoIdleFunction(
  void *lpParameter   // thread data
)
{
  OtxEventDataS eventData;
  OTX_RESULT result;


  while (1) {
    OtxDrvWaitForSingleEvent(g_hNotificationEvent, OTX_TIME_INFINITY);
    //DemoLogf("Event signalled\n");
    do {
      result = OtxDrvGetEventData(g_hEventQueue, &eventData); DemoCheckRc(g_hEventQueue, result);

      if (OTX_RESULT_CODE(result) == OTX_S_OK) {
        // We have a driver event
        // Printf the event data.
        // In a real application branch based on the data
        DemoPrintEvent(&eventData);

      }
    } while (OTX_RESULT_CODE(result) == OTX_S_OK);
  }

  return 0;


}



/*******************************************************************************
  @func
    Prints a timestamp in a MM-DD hh:mm:ss:mmm format, where

    MM  = month
    DD  = day (of month)
    hh  = hour of day
    mm  = minutes of day
    ss  = seconds of day
    mmm = milliseconds

*******************************************************************************/
void DemoPrintTimeStamp(
  OTX_DATETIME nTimeStamp
)
{
  OTX_DATETIME nTimeStampMs;
  OTX_DATETIME nTimeStampSec;
  struct Osa_tm *ptCurrentDayAndTime;
  Osa_time_t nCurrentTimeSecs;
  
  // Subtract number of 100ns ticks between Jan 1, 1970 and Jan 1, 1600
  
  #if defined(__linux__)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 1164447360 * 100000000;  
  #elif defined(WIN32)
    nTimeStamp.nUint64 = nTimeStamp.nUint64 - 116444736000000000;  
  #endif

  
  // Convert to seconds since Jan 1, 1970 (so we can use this time with OsaStdLocalTime())
  nTimeStampSec.nUint64 = nTimeStamp.nUint64 / 10000000;
  
  // Calculate number of milliseconds (since number of seconds is not a very accurate way to report timestamp)
  nTimeStampMs.nUint64 = nTimeStamp.nUint64 / 10000 - (nTimeStampSec.nUint64 * 1000);
  
  nCurrentTimeSecs = nTimeStampSec.sUint32.nLow;
  ptCurrentDayAndTime = OsaStdLocalTime(&nCurrentTimeSecs);
  
  if (ptCurrentDayAndTime) {
    
    DemoLogf("%.2d-%.2d %.2d:%.2d:%.2d.%.3d ", 
      ptCurrentDayAndTime->tm_mon + 1,
      ptCurrentDayAndTime->tm_mday,
      ptCurrentDayAndTime->tm_hour,
      ptCurrentDayAndTime->tm_min, 
      ptCurrentDayAndTime->tm_sec,
      nTimeStampMs.sUint32.nLow
      );
  } else {
    DemoLogf("Error: Invalid timestamp (before Jan 1, 1970)\n");
  }
    
}

/*******************************************************************************
  @func
    Print buffer usage statistics.
*******************************************************************************/
#ifdef DEMO_LINK_WITH_LIBCMT
void DemoStatsThread(
  void *lpParameter   // thread data
)
#else
void *DemoStatsThread(
  void *lpParameter   // thread data
)
#endif
{
  OTX_RESULT result;
  OTX_UINT32 nCurrentUsage;
  OTX_UINT32 nMaxUsage;
  OTX_UINT32 nLostEvents;
  OTX_UINT32 nQueuedCurrent;
  OTX_UINT32 nQueuedPeak;
  OTX_INT32 nDsp;

  while (g_bStatsThreadActive) {
    
    for (nDsp = 0; nDsp < 8; nDsp++) {
      
      //result = OtxDrvGetAttributeValueUint32(g_hPhysDevs.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_SIZE, &nAttribValue); DemoCheckRc(g_hPhysDevs.hDsp[nDsp], result);
      //DemoLogf("  Size (in words)         : %d\n", nAttribValue);
      
      result = OtxDrvGetAttributeValueUint32(g_hPhysDevsVidar.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_CURRENT_USAGE, &nCurrentUsage); DemoCheckRc(g_hPhysDevsVidar.hDsp[nDsp], result);
      //DemoLogf("  Current usage (in words): %d\n", nAttribValue);
      
      result = OtxDrvGetAttributeValueUint32(g_hPhysDevsVidar.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_MAX_USAGE, &nMaxUsage); DemoCheckRc(g_hPhysDevsVidar.hDsp[nDsp], result);
      //DemoLogf("  Max usage (in words)    : %d\n", nAttribValue);
      
      result = OtxDrvGetAttributeValueUint32(g_hPhysDevsVidar.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_BUF_LOST_EVENTS, &nLostEvents); DemoCheckRc(g_hPhysDevsVidar.hDsp[nDsp], result);
      
      result = OtxDrvGetAttributeValueUint32(g_hPhysDevsVidar.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_EXT_QUEUE_CURRENT_USAGE, &nQueuedCurrent); DemoCheckRc(g_hPhysDevsVidar.hDsp[nDsp], result);
      
      result = OtxDrvGetAttributeValueUint32(g_hPhysDevsVidar.hDsp[nDsp], OTX_ATTR_DSP_NOTIFICATION_EVENT_EXT_QUEUE_PEAK_USAGE, &nQueuedPeak); DemoCheckRc(g_hPhysDevsVidar.hDsp[nDsp], result);
      
      DemoLogf("Dsp on Vidar 5x16[%d] curr=%.8d, max=%.8d, lost=%d, qcurr=%d, qpeak=%d\n", nDsp, nCurrentUsage, nMaxUsage, nLostEvents, nQueuedCurrent, nQueuedPeak);
      
    }
    OsaThreadSleep(2000); 
  }

  DemoLogf("DemoStatsThread() terminating.\n");

  return 0;
}



/*******************************************************************************
  @func
    Raw Data Sender Thread Function 
*******************************************************************************/

#ifdef DEMO_LINK_WITH_LIBCMT
void DemoRawDataSender(
  void *lpParameter   // thread data
)
#else
void *DemoRawDataSender(
  void *lpParameter   // thread data
)
#endif
{
  OtxEventDataS eventData;
  OTX_RESULT result;
  OTX_RESULT otxRc;
  OTX_UINT32 i;
  char msgStr[256];
  clock_t startTime, stopTime;
  OTX_UINT32 nSourceId;
  
  startTime = clock();
  nSourceId = *((OTX_INT32 *) lpParameter);
  if (nSourceId >= 8) {
    DemoLogf("DemoRawDataSender: Thread was passed an invalid nSourceId(%d). Thread exiting...\n", nSourceId);
    goto END;
  }
  g_hLogicDevs.mRawDataSender[nSourceId].coUnderRunErrors = 0;
  // Fill a buffer of a constant value to be sent by this sender
  memset(g_hLogicDevs.mRawDataSender[nSourceId].sqTxBuf, nSourceId, sizeof(OTX_UINT8) * DEMO_RAW_TX_BUF_SIZE);
  g_hLogicDevs.mRawDataSender[nSourceId].cbTxBuf = DEMO_RAW_TX_BUF_SIZE;
  g_hLogicDevs.mRawDataSender[nSourceId].cbSent = 0;
  do {
    result = OtxDrvGetEventData(g_hLogicDevs.mRawDataSender[nSourceId].hEventQueue, &eventData); DemoCheckRc(g_hLogicDevs.mRawDataSender[nSourceId].hEventQueue, result);
    
  } while (OTX_RESULT_CODE(result) == OTX_S_OK);
  do {
    otxRc = OtxRawDataSendData(g_hLogicDevs.mRawDataSender[nSourceId].hHandle, g_hLogicDevs.mRawDataSender[nSourceId].sqTxBuf, g_hLogicDevs.mRawDataSender[nSourceId].cbTxBuf, DEMO_TASK_REF, OTX_TIME_INFINITY);
    if (OTX_RESULT_CODE(result) == OTX_S_OK) {
      g_hLogicDevs.mRawDataSender[nSourceId].cbSent += g_hLogicDevs.mRawDataSender[nSourceId].cbTxBuf;
    }
  } while ((OTX_RESULT_CODE(otxRc) != OTX_E_BUSY) && (OTX_RESULT_CODE(otxRc) != OTX_E_NOT_WORKING));
  while (1) {
    OtxDrvWaitForSingleEvent(g_hLogicDevs.mRawDataSender[nSourceId].hNotificationEvent, OTX_TIME_INFINITY);
    do {
      result = OtxDrvGetEventData(g_hLogicDevs.mRawDataSender[nSourceId].hEventQueue, &eventData); DemoCheckRc(g_hLogicDevs.mRawDataSender[nSourceId].hEventQueue, result);
      
      if (OTX_RESULT_CODE(result) == OTX_S_OK) {
        // We have a driver event
        switch (eventData.m_eDeviceType) {
        case OTX_LDEVICE_DATA_RAW_SENDER:
          // Event from Raw Data Sender 
          if ((eventData.m_nSourceId != nSourceId)) {
            DemoLogf("RawDataSender[%d] Error: m_nSourceId is out of range.\n", eventData.m_nSourceId);
            break;
          } else {
            i = eventData.m_nSourceId;
            //DemoLogf(" %d", eventData.m_nSourceId);
          }
          switch (eventData.m_nCode) {
          case OTX_RAWDATA_SENDER_EC_COMPLETED:
            g_hLogicDevs.mRawDataSender[i].coUnderRunErrors++;
            //DemoLogf("DemoRawDataSender[%d]: *** Data Underrun at cbTxSent/cbTxBuf=%d/%d (m_nParam=%d). \n", nCin, g_sqCin[nCin].cbTxSent, g_sqCin[nCin].cbTxBuf, eventData.m_nParam);
            // Fall-through
          case OTX_RAWDATA_SENDER_EC_BUF_DEPLETED:
            {
              OTX_RESULT otxRc;
              OTX_INT32 nCount = 0;
              
              do {
                nCount++;
                otxRc = OtxRawDataSendData(g_hLogicDevs.mRawDataSender[i].hHandle, g_hLogicDevs.mRawDataSender[i].sqTxBuf, g_hLogicDevs.mRawDataSender[nSourceId].cbTxBuf, DEMO_TASK_REF, OTX_TIME_INFINITY);
                if (OTX_RESULT_CODE(otxRc) == OTX_S_OK) {
                  g_hLogicDevs.mRawDataSender[nSourceId].cbSent += g_hLogicDevs.mRawDataSender[nSourceId].cbTxBuf;
                }
                //if (nCount > 1) DemoLogf(".");
              } while ((OTX_RESULT_CODE(otxRc) != OTX_E_BUSY) && (OTX_RESULT_CODE(otxRc) != OTX_E_NOT_WORKING));
              
              OsaThreadSleep(1); // Give the driver some time to empty its queue of data (128ms fills 1024 bytes in a 64kbit/s pipe)
              
            }
            break;
          default:
            otxRc = OtxDrvEventCode2String(eventData.m_hDevice, eventData.m_nCode, 256, &msgStr[0]);
            // Unexpected event from this device
            if (OTX_SUCCESS(otxRc)) {
              DemoLogf("RawDataSender[%d] Event: %s \n", eventData.m_nSourceId, msgStr);
            } else {
              DemoLogf("RawDataSender[%d] Invalid Event: %ld \n", eventData.m_nSourceId, eventData.m_nCode);
              
            }
            break;
          }
          break;
          default:
            // Unhandled event
            DemoLogf("DemoRawDataSender[%d]: Event from an unknown object 0x%X\n", nSourceId, eventData.m_hDevice);
            break;
        }
      }
    } while (OTX_RESULT_CODE(result) == OTX_S_OK);
  }
  
END:
  // We should never get here
  stopTime = clock();
  DemoLogf("DemoRawDataSender[%d]: Done (time=%2.1f seconds, coUnderRunErrors=%d).\n", nSourceId, (double)(stopTime - startTime) / CLOCKS_PER_SEC, g_hLogicDevs.mRawDataSender[nSourceId].coUnderRunErrors);
#ifdef DEMO_LINK_WITH_LIBCMT
  _endthread();
#else
  return 0;
#endif
}


void DemoPrintEvent(
/*******************************************************************************
  @func
    Prints a Driver Event. 
*******************************************************************************/
  OtxEventDataS *pEventData
)
{
  OTX_RESULT otxRc;
  char msgStr[256];

  //EnterCriticalSection(&g_analyzeCritSec);
  DemoPrintTimeStamp(pEventData->m_nTimeStamp);
    
  switch (pEventData->m_eDeviceType) {
    case OTX_DEVICE_LI_T1E1:
    // Event from Li0 or Li1
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      if (pEventData->m_nParam) {
        DemoLogf("E1/T1[%d] Event: %s (%d)\n", pEventData->m_nSourceId, msgStr, pEventData->m_nParam);
      } else {
        DemoLogf("E1/T1[%d] Event: %s\n", pEventData->m_nSourceId, msgStr);
      }
    } else {
      DemoLogf("E1/T1[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;

  case OTX_DEVICE_DSP:
    // Non-logical device related event from the DSPs
    otxRc = OtxDrvEventCode2String(pEventData->m_hDevice, pEventData->m_nCode, 256, &msgStr[0]);
    if (OTX_SUCCESS(otxRc)) {
      DemoLogf("DSP[%d] Event: %s \n", pEventData->m_nSourceId, msgStr);
    } else {
      DemoLogf("DSP[%d] Invalid Event: %ld \n", pEventData->m_nSourceId, pEventData->m_nCode);
    }
    break;
    
  default:
    // Unhandled event
    DemoLogf("Event from an unknown object 0x%X\n", pEventData->m_hDevice);
    break;
  }

}

OTX_INT32 DemoProcessUserCommand(char commandChar)
/*******************************************************************************

  @func

  Processes User Commands

  @comm

*******************************************************************************/
{
  static OTX_INT32 nCurrentLi = 0;
  OTX_RESULT result;

  DemoLogf("Key '%c' pressed\n", commandChar);

  switch (commandChar) {

  case '?':
  case 'h':
  case 'H':
    DemoLogf("Cmd  Synopsis\n");
    DemoLogf("===  ========\n");
    DemoLogf(" ?   Display Help text\n");
    DemoLogf(" h   Ditto\n");
    DemoLogf(" q   Quit\n");
    DemoLogf(" n   Select active Li or Dsp, this Li/Dsp is then used for subsequent commands\n");
    DemoLogf(" x   Send a text file using the RawData Sender[%d]. Received by the RawData Receiver[%d]\n", nCurrentLi, !nCurrentLi);
    DemoLogf(" a   Abort the Raw Data Sender[%d]\n", nCurrentLi);
    DemoLogf(" s   Start Raw Data Sender[%d]. Prompt for file to send\n", nCurrentLi);
    DemoLogf(" S   Start Raw Data Sender[%d]. Continously send a text string\n", nCurrentLi);
    DemoLogf(" l   Query Line Status of both Lis\n");
    DemoLogf(" t   Toggle verbose buffer usage statistics\n");
    break;

  case 't':
    {
      g_bStatsThreadActive = !g_bStatsThreadActive;

      if (g_bStatsThreadActive) {
        DemoLogf("Starting Stats Thread\n");
        OsaThreadStart(DemoStatsThread);
      }

    }
    break;




#if 1 // Only for debugging
  case 'b':
    {
      char szBuf[12];
      OTX_INT32 nDsp;
      
      DemoLogf("Init Bsp. Which Dsp#?");
      OsaConsoleGetStr(szBuf);
      nDsp = (OTX_INT32) strtol(szBuf, NULL, 0);
      
      OtxDrvDbgDump(g_hPhysDevsVidar.hDsp[nDsp], 1);

    }
    break;
  case 'B':
    {
      OTX_INT32 nDsp;
      
      for (nDsp = 0; nDsp < DEMO_DSP_MAX; nDsp++) {
        DemoLogf("Init Bsp for Dsp#%d\n", nDsp);
        OtxDrvDbgDump(g_hPhysDevsVidar.hDsp[nDsp], 1);
      }
    }
    break;

  case 'f':
    {
      OTX_INT32 nLi;
        for (nLi = 0; nLi < 8; nLi++) {
          DemoLogf("Forcing resynchronization on Li #%d... ", nLi);
          result = OtxT1E1ForceResynch(g_hPhysDevs.hLi[nLi]); DemoCheckRc(g_hPhysDevs.hLi[nLi], result);
          DemoLogf("Done.");
        }
      
    }
    break;

  case 'F':
    {
      OTX_INT32 nLi;
        for (nLi = 0; nLi < 8; nLi++) {
          DemoLogf("Reseting Li #%d... ", nLi);
          result = OtxDrvReset(g_hPhysDevs.hLi[nLi]); DemoCheckRc(g_hPhysDevs.hLi[nLi], result);
          DemoLogf("Done.");
        }
      
    }
    break;
#endif


  case 'x':
    {
      //OTX_RESULT otxRc = OTX_S_OK;
      char sqzFileName[64];

      if ((g_hLogicDevs.mRawDataSender[nCurrentLi].pTxFile = fopen(sqzFileName, "rb")) == NULL) {
        DemoLogf("Error: Could not open file %s \n", sqzFileName);
        break;
      }

      DemoLogf("Creating Raw Data Sender[%d] Thread\n", nCurrentLi);
#ifdef DEMO_LINK_WITH_LIBCMT
      _beginthread(DemoRawDataSender, 0, (void *) &g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#else
    	OsaThreadStartEx(DemoRawDataSender, (void *)&g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#endif
      
    
    }
    break;


  case 'a':
    {

      DemoLogf("Aborting Raw Data Sender[%d] Thread\n", nCurrentLi);
      g_hLogicDevs.mRawDataSender[nCurrentLi].bActive = OTX_FALSE;

    }
    break;

  case 's':
    {
      //OTX_RESULT otxRc = OTX_S_OK;
      char sqzFileName[64];

      DemoLogf("RawDataSender[%d]. Please enter file to send: ", nCurrentLi);
      OsaConsoleGetStr(sqzFileName);

      if ((g_hLogicDevs.mRawDataSender[nCurrentLi].pTxFile = fopen(sqzFileName, "rb")) == NULL) {
        DemoLogf("Error: Could not open file %s \n", sqzFileName);
        break;
      }

      DemoLogf("Creating Raw Data Sender[%d] Thread\n", nCurrentLi);
#ifdef DEMO_LINK_WITH_LIBCMT
      _beginthread(DemoRawDataSender, 0, (void *) &g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#else
    	OsaThreadStartEx(DemoRawDataSender, (void *)&g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#endif
    
    }
    break;



  case 'S':
    {
      DemoLogf("RawDataSender[%d]. Continously send text string \"%s\".\n", nCurrentLi, DEMO_TEST_STRING);

      g_hLogicDevs.mRawDataSender[nCurrentLi].pTxFile = NULL;

      DemoLogf("Creating Raw Data Sender Thread for Line# %d\n", nCurrentLi);
      
#ifdef DEMO_LINK_WITH_LIBCMT
      _beginthread(DemoRawDataSender, 0, (void *) &g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#else
    	OsaThreadStartEx(DemoRawDataSender, (void *)&g_hLogicDevs.mRawDataSender[nCurrentLi].nSourceId);
#endif
      
    
    }
    break;



  case 'l':
    {
      OtxT1E1Layer1StatusE eLayer1Status;
      OTX_INT32 nLi;
      OTX_INT32 iBrd;
      for (iBrd = 0; iBrd < g_demoEnv.coBrds; iBrd++){
        for (nLi = 0; nLi < 8; nLi++) {
          DemoLogf("Line status of Li #%d is ", nLi);
          result = OtxT1E1GetLayer1Status(g_hPhysDevs.hLi[nLi], &eLayer1Status); DemoCheckRc(g_hPhysDevs.hLi[nLi], result);
          DemoLogf("%s\n", eLayer1Status == OTX_T1E1_LAYER1_OK ? "OK" : "DOWN");
        }
      }
    }
    break;

  case 'n':
    {
      // Select current nCurrentLi
      nCurrentLi++;
      if (nCurrentLi > 7) {
        // Wrap 
        nCurrentLi = 0;
      }
      
      DemoLogf("RawData Sender #%d Selected \n", nCurrentLi);
      
    }
    break;
    
    
  case 'N':
    {
      // Select current nCurrentLi
      nCurrentLi--;
      if (nCurrentLi < 0) {
        // Wrap
        nCurrentLi = 7;
      }
      
      DemoLogf("RawData Sender #%d Selected \n", nCurrentLi);
    }
    break;
    
  case 'q':
  case 'Q':
    return 1;
    break;

  default:
    DemoLogf("Unknown command. Press '?' for help, 'q' to quit\n");
    break;
  }

  return 0;
}                      


/*******************************************************************************

  @func

  Parse the program's command line arguments and calls upon
  requested actions.

  There are some optional switches that can be used following
  the driver name on the command line. Switches are are
  separated by spaces, and can appear in arbitrary order.
  Switches consist of a hyphen and a letter:

  @comm

*******************************************************************************/
static void DemoParseCmdLineArgs(int argc, char *argv[], DemoEnvT *envP) {
  short  i;

  // Set reasonable defaults before reading from command line
  envP->bExitOnError      = OTX_FALSE;
  envP->logFileName       = DEMO_DEFAULT_LOG_FILE;
  envP->law               = OTX_DATA_FORMAT_COMPAND8_U_LAW;
  envP->eT1E1Mode         = OTX_T1E1_MODE_E1;
  envP->coLi              = 8;
  envP->coCtBuses         = 1;
  envP->coBrds            = DEMO_DEFAULT_NO_OF_T8P_BRDS;
  envP->nRawDataCapacity  = 64; //default rawdata capacity
  envP->nStartingTimeslot = 1;


  for (i = 1; i < argc; i++) {        // Scan command line arguments
    OsaStdStrUpr(argv[i]);
    if (*argv[i] == '-' || *argv[i] == '/') {  // Is argument a legal switch?
      if (*(argv[i]+1) == 'L') {   // Log file
        envP->logFileName = argv[i]+2;
      } else if (*(argv[i]+1) == 'B') {   // Number of T8P boards
        envP->coBrds = strtoul(argv[i]+2, NULL, 0);
        
      } else if (*(argv[i]+1) == 'S') {   // Starting timeslot
        envP->nStartingTimeslot = strtol(argv[i]+2, NULL, 0);

      } else if (*(argv[i]+1) == 'C') {   // Companding Law
        if (*(argv[i]+2) == 'A') {   // A-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_A_LAW;
        } else if (*(argv[i]+2) == 'U') {   // U-law
          envP->law = OTX_DATA_FORMAT_COMPAND8_U_LAW;
        } else {
          DemoLogf("Error: Invalid companding law command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
          exit(1);
        }
      } else if (*(argv[i]+1) == 'M') {   // Li Mode
        if (*(argv[i]+2) == 'E') {   // E1 
          envP->eT1E1Mode    = OTX_T1E1_MODE_E1;
        } else if (*(argv[i]+2) == 'T') {   // T1
          envP->eT1E1Mode    = OTX_T1E1_MODE_T1;
        } else {
          DemoLogf("Error: Invalid Li Mode (E1/T1) command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
          exit(1);
        }
      } else if (*(argv[i]+1) == 'N') { // Capacity of Raw Data Pipe
        envP->nRawDataCapacity = strtol(argv[i]+2, NULL, 0);
        if (envP->nRawDataCapacity >= 64) {
          if ((envP->nRawDataCapacity%64) != 0) {
            DemoLogf("Error: Raw Data Pipe capacity must be a multiple of 64\n");
            DemoHelpPanel();
            OsaConsoleExit();
            exit(1);
          }
        } else {
        DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
        DemoHelpPanel();
        OsaConsoleExit();
        exit(1);
       }

      } else {
          DemoLogf("Error: Unknown command line switch %s\n", argv[i]);
          DemoHelpPanel();
          OsaConsoleExit();
          exit(1);
        }
      }
    }
  
}


/*******************************************************************************

  @func

  Displays a help for the command line.

  @comm

*******************************************************************************/
static void DemoHelpPanel()
{
  DemoLogf("\nCommand line switches:\n");
  DemoLogf(" /L<fileName> : Log file name\n");
  DemoLogf(" /B<boardNo>  : Number of Thor-8-PCI boards (Default is %d)\n", DEMO_DEFAULT_NO_OF_T8P_BRDS);
  DemoLogf("                  Used with Vidar-5x16-ASM\n");
  DemoLogf(" /CA          : A-law\n");
  DemoLogf(" /CU          : u-law\n");
  DemoLogf(" /N<capacity> : Raw Data Pipe capacity in kbps (in 64kbps increments. max capacity 1984kbps)\n");
  DemoLogf(" /ME          : E1 Li mode\n");
  DemoLogf(" /MT          : T1 Li mode\n");
  DemoLogf(" /S<timeslot> : Starting timeslot (default is TS1)\n");
}


/*******************************************************************************

  @func

  Displays a message on the screen and prints it to the logfile.

  @comm

*******************************************************************************/
static void DemoLogf(char *formatStr, ...)
{
  va_list argList;
  va_start(argList, formatStr);
  OsaConsolePrintF(formatStr, argList);
  if (g_pDemoLogFile) vfprintf(g_pDemoLogFile,   formatStr, argList);
  va_end(argList);
}



int main(int argc, char *argv[])
/*******************************************************************************
*******************************************************************************/
{
  OTX_INT32 exitFlag = 0;
  OTX_RESULT result;  // OTX Standard Return Code
  char commandChar;
  OTX_INT32 iBrd;
	OsaConsoleInit();

  printf("T8MRAWDATA with Vidar5x16, %s, Odin TeleSystems Inc.\n", __DATE__);
  printf("Vidar5x16 Example program for Raw Data Sender/Receiver.\n");
  DemoParseCmdLineArgs(argc, argv, &g_demoEnv);
  if ((g_pDemoLogFile = fopen(g_demoEnv.logFileName, "wt")) == NULL) {
    printf("Couldn't open logfile %s\n", g_demoEnv.logFileName);
    OsaConsoleExit();
    exit(-1);
  }
  DemoLogf("Log file is %s\n", g_demoEnv.logFileName);
  DemoLogf("Raw Data Pipe capacity is %d\n", 1984);
  DemoLogf("Number of CT Buses : %d\n", g_demoEnv.coCtBuses);
  DemoLogf("Number of boards   : %d\n", g_demoEnv.coBrds);
  // Query Library revision
  {
    char szVersionInfo[255];
    OTX_BOOL bIsStaticLinkedLibrary;
    char szLibFileName[64];
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_REV_STR, szVersionInfo, 255); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueBool(OTX_LIB_HANDLE, OTX_ATTR_LIB_TYPE_STATIC, &bIsStaticLinkedLibrary); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    result = OtxDrvGetAttributeValueSz(OTX_LIB_HANDLE, OTX_ATTR_LIB_FILE_NAME, szLibFileName, 64); DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    
    DemoLogf("Library version %s (filename %s, %s linked)\n", szVersionInfo, szLibFileName, bIsStaticLinkedLibrary ? "static":"dynamic");
  }
  // Connect to the Driver
  DemoLogf("Connecting lib...\n");
  result = OtxDrvConnectLib(); 
  switch (OTX_RESULT_CODE(result)) {
  case OTX_RES_HANDLE:
    DemoLogf("Error: Could not connect to the OTX library.\nPossible cause: OTX Driver not loaded\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  case OTX_RES_NOT_SUPPORTED:
    DemoLogf("Error: Mismatch between OTX Driver revision and OTX Library revision.\nRevisions must match.\n");
    OsaConsoleExit();
    OtxDrvReleaseLib();
    exit(1);
    break;
  default:
    DemoCheckRc(OTX_INVALID_HANDLE_VALUE, result);
    break;
  }
  // Create Event Queues for Driver --> Application Communications
  DemoLogf("DemoCreateEventQueue\n");
  DemoCreateEventQueue();
  // Connect to the Physical CT Buses
  {
    OTX_INT32 nCtBus;
    for (nCtBus = 0; nCtBus < g_demoEnv.coCtBuses; nCtBus++) {
      DemoLogf("Open OTX_DEVICE_CT_BUS[%d]\n", nCtBus);
      result = OtxDrvOpenPhysicalDevice(0, OTX_DEVICE_CT_BUS, nCtBus, nCtBus, g_hEventQueue, &g_hCtBus[nCtBus]); DemoCheckRc(g_hCtBus[nCtBus], result);
      // Limit the number of highways that OTX can use on this CT Bus (only to demonstrate this functionality)
      result = OtxDrvSetAttributeValueUint32(g_hCtBus[nCtBus], OTX_ATTR_CTBUS_OTX_HWYS, 4); DemoCheckRc(g_hCtBus[nCtBus], result);
      // Limit the number of timeslots that OTX can use on this CT Bus (only to demonstrate this functionality)
      result = OtxDrvSetAttributeValueUint32(g_hCtBus[nCtBus], OTX_ATTR_CTBUS_OTX_TIMESLOTS_PER_HWY, 16); DemoCheckRc(g_hCtBus[nCtBus], result);
    }
  }
  for (iBrd = 0; iBrd < DEMO_DEFAULT_NO_OF_T8P_BRDS; iBrd++) {
    // Connect (get handles) to the physical devices on the Thor-8-PCI board
    DemoLogf("Opening physical devices...\n");
    DemoOpenPhysicalDevices();
  }
  DemoLogf("Opened Thor-8-PCI board\n");
  //Connect (get handles) to the physical devices on the Vidar 5x16 PCI board
  DemoOpenPhysicalDevicesVidar();
  //Inform the driver that one Thor-8-pci board is connected to the CT Bus
  result = OtxDrvConnectBoard(g_hCtBus[0], g_hPhysDevs.hThorController); DemoCheckRc(g_hCtBus[0], result);
  //Inform the driver that one vidar5x16 board is connected to the CT Bus
  result = OtxDrvConnectBoard(g_hCtBus[0], g_hPhysDevsVidar.hVidarController); DemoCheckRc(g_hCtBus[0], result);
  // Configure E1/T1 devices
  DemoLogf("Configuring E1/T1 devices...\n");
  DemoConfigureE1T1Devices();
  DemoLogf("Configured E1/T1 devices\n");
  // Enable physical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoLogf("DemoEnablePhysicalDevices\n");
  DemoEnablePhysicalDevices();
  //Enable Vidar 5x16 physical board
  DemoLogf("DemoEnablePhysicalDeviceVidar\n");
  DemoEnablePhysicalDevicesVidar();
  // Set clock source for the Thor-8-PCI board
  DemoLogf("Setting Clock Sources for the Thor-8-PCI Boards\n");
  // Thor-8-PCI Board 0 is the H.100 Clock master
  result = OtxBrdSetClocks(g_hPhysDevs.hThorController, OTX_CLOCK_MODE_H100, OTX_CLOCK_SOURCE_LOCAL_0); DemoCheckRc(g_hPhysDevs.hThorController, result);
  // Set clock source for the Vidar-5x16-PCI board (H.100 Clock slave)
  result = OtxBrdSetClocks(g_hPhysDevsVidar.hVidarController, OTX_CLOCK_MODE_H100, OTX_CLOCK_SOURCE_A_CLOCKS); DemoCheckRc(g_hPhysDevsVidar.hVidarController, result);
  // Create and get handles to the Logical devices needed by this application
  //Create Logical devices for each of the Thor-8-PCI boards
  DemoLogf("DemoCreateLogicalDevices\n");
  DemoCreateLogicalDevices();
  // Create and get handles to the Pipe devices needed by this application
  DemoLogf("DemoConnectDevices for Board [%d]\n", iBrd);
  DemoConnectDevices();
  // Initialize the logical devices. A logical device must be create and connected before it can be initialized.
  DemoLogf("DemoInitializeLogicalDevices\n");
  DemoInitializeLogicalDevices();
  // Enable logical devices. Their device state should now be OTX_DEVICE_WORKING
  DemoLogf("DemoEnableLogicalDevices\n");
  DemoEnableLogicalDevices();
  // Main Loop. Loop here and wait for Events from the driver or commands from the user ....
	OsaThreadStart(DemoIdleFunction);
  OsaThreadSleep(300); // Only so the following message will show on the screen
  DemoLogf("\nPress 'h' for help\n");
  while (!exitFlag) {
    commandChar = getch();
    exitFlag = DemoProcessUserCommand(commandChar);
  }
  // Clean up
  OsaConsoleExit();
  OtxDrvReleaseLib();
  return 0;
}

