/*******************************************************************************

  @doc PublicHdlcExLibApi OtxHdlcEx

  Copyright (C) 2004 Odin TeleSystems Inc., All Rights Reserved

  OTX Generic HdlcEx Library API

  @module
  OtxHdlc | Prototypes for the Logical HDLC Device (Sender/Receiver) Functions

  @contents2
  OtxHdlcX.h |

  @index func | OtxHdlcX

  @contents2
  OTX Logical Hdlc Sender/Receiver Functions - OtxHdlcExXXX() | The API functions for the Software High-level Data Link Controller (HDLC) device

  @head3 OtxHdlcExXXX() Function Reference |

  @index func | OtxHdlcEx

*******************************************************************************/

#ifndef OTXHDLCEX__LIB__H
#define OTXHDLCEX__LIB__H

#include "OtxType.h"
#include "OtxErr.h"
#include "OtxHdlcD.h"
#include "OtxDev.h"


#if defined(__linux__)
  #define OTXHDLCEX_API

#elif defined(WIN32)
  #ifdef OTXHDLCEXDLL_EXPORTS
    #define OTXHDLCEXDLL_API __declspec(dllexport)
  #else
    #define OTXHDLCEXDLL_API __declspec(dllimport)
  #endif
#else
  #error "Platform is not supported."
#endif



#ifndef HDLCX_API
  #define HDLCX_API OTXHDLCEXDLL_API
#endif

typedef OTX_UINT32 HDLCX_HANDLE;

#define HDLCX_INVALID_HANDLE_VALUE 0xFFFFFFFF


#ifdef __cplusplus
  extern "C"{
#endif

// Library level access functions
OTX_RESULT HDLCX_API OtxHdlcExConnectLib();
OTX_RESULT HDLCX_API OtxHdlcExReleaseLib();



/*******************************************************************************

  @func
     Creates software HDLC-device 
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExCreate(
  IN  OtxLogicalDeviceTypeE eDevType,   // @parm Class of device we are looking for, as known by the module in question. See also <t OtxLogicalDeviceTypeE>
  IN  OTX_HANDLE      hEventQueue,      // @parm Handle to separate event queue (0 creates device specific event queue)
  IN  OTX_INT32       nEventSourceId,   // @parm User specified ID to be used by this device as the Source ID for the events generated by the device
  OUT HDLCX_HANDLE   *phLogicDevice    // @parm Handle to the new logical device (if result code OTX_S_OK)
);


/*******************************************************************************

  @func
     Destroy HDLC-device 
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExClose(
  IN HDLCX_HANDLE     hDevice                 // @parm Handle to the HdlcEx device
);


/*******************************************************************************

  @func
     Enable HDLCC-device 
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExEnable(
  IN HDLCX_HANDLE     hDevice                 // @parm Handle to the HdlcEx device
);


/*******************************************************************************

  @func
     Disable HDLC-device 
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExDisable(
  IN HDLCX_HANDLE     hDevice                 // @parm Handle to the HdlcEx device
);


/*******************************************************************************

  @func
     Reset HDLC-device 
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExReset(
  IN HDLCX_HANDLE     hDevice                 // @parm Handle to the HdlcEx device
);


/*******************************************************************************

  @func
    Initializate the HDLC Sender.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>.

  @ex |

      result = OtxHdlcExSenderInit(
	                g_hLogicDevs.hHdlcSender,
	                OTX_HDLC_FFT_FLAGS,
	                2,
	                OTX_HDLC_CRC16,
	                OTX_FALSE,
	                OTX_TRUE,
	                OTX_HDLC_TR_NA);


  @xref
    <nl><f OtxHdlcExReceiverInit>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSenderInit(
  IN HDLCX_HANDLE     hDevice,                // @parm Handle to the HdlcEx device
  IN OtxHdlcFrameFillTypeE  eFrameFillType,   // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32        nInterFrameTimeFill,    // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE  eCrcMode,               // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL         bBitInversion,          // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL         bAppendTxCrc,           // @parm If OTX_TRUE the HDLC device will append the CRC to the frame to be transmitted. If OTX_FALSE, the CRC is generated with software
  IN OtxHdlcTxRateE   eTxRate                 // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
);

/*******************************************************************************

  @func
    Initializate the HDLC Receiver.

  @comm
    Start HDLC operation with by calling the function <f OtxDrvEnable> and stop
    it by calling the function <f OtxDrvDisable>
  
  @ex |

      // initialize the HDLC Receiver
      result = OtxHdlcExReceiverInit(
	                g_sqHdlcReceiver[i].hHandle,
	                OTX_HDLC_FFT_FLAGS,
	                2,
	                OTX_HDLC_CRC16,
	                OTX_FALSE,
	                OTX_TRUE,
	                OTX_HDLC_TR_NA,
	                OTX_FALSE);


  @xref
    <nl><f OtxExHdlcSenderInit>
    <nl><e OtxLogicalDeviceTypeE.OTX_LDEVICE_HDLC_SENDER>
    <nl><t OtxHdlcFrameFillTypeE>
    <nl><t OtxHdlcCrcModeE>
    <nl><t OtxHdlcTxRateE>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExReceiverInit(
  IN HDLCX_HANDLE     hDevice,                // @parm Handle to the HdlcEx device
  IN OtxHdlcFrameFillTypeE  eFrameFillType,   // @parm Fill character(s) to be send between HDLC frames. See also <t OtxHdlcFrameFillTypeE>
  IN OTX_INT32        nInterFrameTimeFill,    // @parm Minimum No of interframe time characters between 2 HDLC frames (1=shared flags, 2=non-shared) Min value is 1. Max is 1024.
  IN OtxHdlcCrcModeE  eCrcMode,               // @parm Cyclic Redundancy Check (CRC) mode: 16-bit, 32-bit, or none. See also <t OtxHdlcCrcModeE>
  IN OTX_BOOL         bBitInversion,          // @parm If OTX_TRUE then all bits are inverted when transmitted and before received
  IN OTX_BOOL         bAppendTxCrc,           // @parm If OTX_TRUE the HDLC device will append the CRC to the frame to be transmitted. If OTX_FALSE, the CRC is generated with software
  IN OtxHdlcTxRateE   eTxRate,                // @parm HDLC Transmission Rate for V.110 and X.30 modes. See also <t OtxHdlcTxRateE>
  IN OTX_BOOL         bEveryEventFlag         // @parm Use one event for one frame, otherwise sgenerate only first
);


/*******************************************************************************

  @func
    Filters particular timeslots in data IO buffer used <f by OtxHdlcExProcessIO>

  @comm
    Use it instead of <f OtxDrvConnectPipe> in HW OTX API. 
  @notes
    For solid IO buffers i.e. contained only one timeslot data call it with 
    parameters: coTimeSlotMask=1 and TimeSlotMask[0]=0xff

  @ex |
        OTX_UINT8 TimeSlotMask[32];
        
        memset(TimeSlotMask,0,sizeof(TimeSlotMask);
        TimeSlotMask[0] = 0xff;
        
        result = OtxHdlcExSetTimeSlotMask(
                   g_sqHdlcReceiver[i].hHandle, 
                   sizeof(TimeSlotMask),
                   &TimeSlotMask[0]);
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSetTimeSlotMask(
  IN HDLCX_HANDLE     hDevice,                // @parm Handle to the HdlcEx device
  IN OTX_UINT32       coTimeSlotMask, 
  IN OTX_UINT8       *pnTimeSlotMask
);


// @type pHdlcExCallback | Callback function pointer for HDLCEX
typedef void (*pHdlcExCallback)(OtxEventDataS *);

/*******************************************************************************

  @func
    Use a callback routine in lieu of using an event queue for HDLCEX 
    notifications.

  @comm
    Use it instead of specifying an hEventQueue when calling OtxHdlcExCreate

  @notes
    This enhances performance in high load situations.  The callback is called
    whenever an HDLC frame is ready.

  @ex |

    ...
    result = OtxHdlcExSetCallback(g_sqHdlcReceiver[i].hHandle, MyCallback);
    ...

    pHdlcExCallback MyCallback(OtxEventDataS *pEventData)
    {
      OTX_RESULT            result;
      OTX_UINT8             buf[0x3000];
      OTX_UINT32            nFrameLen;
      OTX_UINT32            nFrameSeq;
      OtxHdlcFrameEndCodeE  nFrameEndCode;
      OTX_DATETIME          nTimeStamp;

      otxRc = OtxHdlcExReadData(pEventData->m_hDevice, buf, sizeof(buf), &nFrameLen, &nFrameSeq, &nFrameEndCode, &nTimeStamp);
  }

  @xref
    <nl><t pHdlcExCallback>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSetCallback(
  IN HDLCX_HANDLE     hDevice,              // @parm Handle to the HdlcEx device
  IN pHdlcExCallback  pCallback             // @parm Pointer to callback function. See <t pHdlcExCallback>
);


// @type pHdlcExCallbackEx | Extended callback function pointer for HDLCEX
typedef void (*pHdlcExCallbackEx)(OtxEventDataS *, void *);
/*******************************************************************************

  @func
    Use a callback routine in lieu of using an event queue for HDLCEX 
    notifications.  This routine allows for an additional context to be
    passed back to the callback.

  @comm
    Use it instead of specifying an hEventQueue when calling OtxHdlcExCreate

  @notes
    This enhances performance in high load situations.  The callback is called
    whenever an HDLC frame is ready.

    typedef void (*pHdlcExCallbackEx)(OtxEventDataS *, void *);

  @ex |

  void ReceiverCallbackEx(OtxEventDataS *pEventData, void *pContext);

  class MyReceiver: public:

  private
     friend void ReceiverCallbackEx(OtxEventDataS *pEventData, void *pContext);

  protected:
     void MyCallback(OtxEventDataS *pEventData);

     HDLCX_HANDLE hHandle;
  };


  void _cdecl ReceiverCallbackEx(OtxEventDataS *pEventData, void *pContext)
  {
     MyReceiver *pMyReceiver = (MyReceiver *) pContext;
     pMyReceiver->MyCallback(pEventData);
  }


  void MyReceiver::Init()
  {
     OTX_RESULT result;
     
     result = OtxHdlcExCreate(OTX_LDEVICE_HDLC_RECEIVER, OTX_INVALID_HANDLE_VALUE, 0, &hHandle);
     result = OtxHdlcExSetCallbackEx(hHandle, ReceiverCallbackEx, (void *) this);
  }

  
  void MyReceiver::MyCallback(OtxEventDataS *pEventData)
  {
    OTX_RESULT            result;
    OTX_UINT8             buf[0x3000];
    OTX_UINT32            nFrameLen;
    OTX_UINT32            nFrameSeq;
    OtxHdlcFrameEndCodeE  nFrameEndCode;
    OTX_DATETIME          nTimeStamp;

    assert(pEventData->m_hDevice == hDevice);
    otxRc = OtxHdlcExReadData(pEventData->m_hDevice, buf, sizeof(buf), &nFrameLen, &nFrameSeq, &nFrameEndCode, &nTimeStamp);
  }


  @xref
    <nl><f OtxHdlcExSetCallback>
    <nl><t pHdlcExCallbackEx>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSetCallbackEx(
  IN HDLCX_HANDLE       hDevice,              // @parm Handle to the HdlcEx device
  IN pHdlcExCallbackEx  pCallbackEx,          // @parm Pointer to callback function. See <t pHdlcExCallbackEx>
  IN void *             pContext              // @parm Context to be passed to the callback
);


// -------------- Device runtime functions 

/*******************************************************************************

  @func
     Sends HDLC data via HDLC-sender device 
     
  @ex 
      for (i = 0; i < 10; i++) {
        arMsg[i] = (OTX_UINT8) i;
      }

      // send one HDLC Frame
      result = OtxHdlcExSendData(
                  g_hLogicDevs.hHdlcSender, 
                  arMsg, 
                  10, 
                  DEMO_TASK_REF, 
                  OTX_TIME_INFINITY); 

     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSendData(
  IN HDLCX_HANDLE     hDevice,                // @parm Handle to the HdlcEx device
  IN OTX_UINT8        *sqData,                // @parm Data to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data to be send
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
     Read reveived HDLC data from HDLC-receiver
     
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExReadData(
  IN HDLCX_HANDLE     hDevice, 
  OUT OTX_UINT8       *pDataBuf,              // @parm Data buffer for the received data
  IN  OTX_UINT32      nBufLen,                // @parm Length of the user data buffer
  OUT OTX_UINT32      *pnFrameLen,            // @parm Length of the received HDLC frame
  OUT OTX_UINT32      *pnFrameSeq,            // @parm Sequence number of the received HDLC frame (matches the number in the event queue)
  OUT OtxHdlcFrameEndCodeE  *pnFrameEndCode,  // @parm Frame Status Code
  OUT OTX_DATETIME    *pnTimeStamp            // @parm Timestamp of the last received octet of the frame
);


/*******************************************************************************

  @func
     IO Process function

  @comm 
     Need to call it both for sender and receiver. Because of Soft devices 
     works with data buffers only, not with HW-devices.
*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExProcessIO(
  IN HDLCX_HANDLE     hDevice,              // @parm Handle to the HDLCEX device
  IN OTX_UINT8*       pDataBuf,             // @parm Data buffer
  IN OTX_UINT32       nBufLen,              // @parm Amount/Size of data buffer
  OUT OTX_UINT32*     nBufUsed              // @parm Upon return, amount of buffer used or filled
);


/*******************************************************************************

  @func
    IO Process Extended function

  @comm 
    Same as OtxHdlcExProcessIO(), except additional parameters are given
    to be used.

    This routine accepts frame reference data.

  @xref
    OtxHdlcExProcessIO

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExProcessIOEx(
  IN HDLCX_HANDLE     hDevice,              // @parm Handle to the HDLCEX device
  IN OTX_UINT8*       pDataBuf,             // @parm Data buffer
  IN OTX_UINT32       nBufLen,              // @parm Amount/Size of data buffer
  IN OTX_UINT64       nFrameRefId,          // @parm Reference ID of first frame in the buffer
  IN OTX_DATETIME     dtFrameRefClock,      // @parm Reference Time of first frame in the buffer
  OUT OTX_UINT32*     nBufUsed              // @parm Upon return, amount of buffer used or filled
);


/*******************************************************************************

  @func
     Bulk IO Process function

  @comm 
     Given an array of device handles, a single bulk buffer is processed.
     This routine can be called in lieu of calling OtxHdlcExProcessIO() for
     each device handle that contains data within a single buffer (example:
     data buffer read from Burst Device will contain data for many devices).

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExProcessIOBulk(
  IN HDLCX_HANDLE     *phDevice,            // @parm pointer to array of handles to HDLCEX devices
  IN OTX_UINT32       phDeviceSize,         // @parm size of handle array
  IN OTX_UINT32       nBufFrameLen,         // @parm Size of a single frame of the data buffer
  IN OTX_UINT8*       pDataBuf,             // @parm Data buffer
  IN OTX_UINT32       nBufLen,              // @parm Amount/Size of data buffer (multiple of phDeviceSize)
  OUT OTX_UINT32*     nBufUsed              // @parm Upon return, amount of buffer used or filled
);


/*******************************************************************************

  @func
     Bulk IO Process Extended function

  @comm 
     Given an array of device handles, a single bulk buffer is processed.
     This routine can be called in lieu of calling OtxHdlcExProcessIO() for
     each device handle that contains data within a single buffer (example:
     data buffer read from Burst Device will contain data for many devices).

     This routine accepts frame reference data.

  @xref
    OtxHdlcExProcessIO

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExProcessIOBulkEx(
  IN HDLCX_HANDLE     *phDevice,            // @parm pointer to array of handles to HDLCEX devices
  IN OTX_UINT32       phDeviceSize,         // @parm size of handle array
  IN OTX_UINT32       nBufFrameLen,         // @parm Size of a single frame of the data buffer
  IN OTX_UINT8*       pDataBuf,             // @parm Data buffer
  IN OTX_UINT32       nBufLen,              // @parm Amount/Size of data buffer (multiple of nBufFrameLen)
  IN OTX_UINT64       nFrameRefId,          // @parm Reference ID of first frame in the buffer
  IN OTX_DATETIME     dtFrameRefClock,      // @parm Reference Time of first frame in the buffer
  OUT OTX_UINT32*     nBufUsed              // @parm Upon return, amount of buffer used or filled
);


//------------------------ SS7 runtime functions

/*******************************************************************************

  @func
    Get the status and statistics of the incoming SS7 link on a specified pipe

  @ex |

      OTX_UINT32  nFisuCount;   // Number of FISUs received
      OTX_UINT32  nLssuCount;   // Number of LSSUs received
      OTX_UINT32  nMsuCount;    // Number of MSUs received
      OtxHdlcSS7FisuS fisu;     // The last received FISU on the pipe.

      result = OtxHdlcExSS7GetReceiveStatus(
                  hHandle, 
                  &nFisuCount, 
                  &nLssuCount, 
                  &nMsuCount, 
                  &fisu); 

      // print out status
      if (OTX_SUCCESS(result)) {
        DemoLogf("FISUs received: %lu\n", nFisuCount);
        DemoLogf("LSSUs received: %lu\n", nLssuCount);
        DemoLogf("MSUs  received: %lu\n", nMsuCount);
        DemoLogf("Last FISU received: bsn = %d, bib = %d, fsn = %d, fib = %d, 
                  li = %d\n", fisu.bsn, fisu.bib, fisu.fsn, fisu.fib, fisu.li);
      }    

  @xref
    <f OtxHdlcExSS7SetFilter>
    <nl><f OtxHdlcExSS7GetFilter>
    <nl><t OtxHdlcExSS7FisuS>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSS7GetReceiveStatus(
  IN HDLCX_HANDLE     hHdlcDevice,            // @parm Handle to the HDLC Device
  OUT OTX_UINT32      *pnFisuCount,           // @parm Number of FISUs received
  OUT OTX_UINT32      *pnLssuCount,           // @parm Number of LSSUs received
  OUT OTX_UINT32      *pnMsuCount,            // @parm Number of MSUs received
  OUT OtxHdlcSS7FisuS *pFisu                  // @parm The last received FISU on the pipe. See also <t OtxHdlcSS7FisuS>
);


/*******************************************************************************

  @func
    Set the filter mask for the filtering out FISUs, LSSUs, and/or MSUs

  @comm 

    It should be used after HDLC receiver device is initialized

  @ex |

    // set filter mask on 2 DSP
    
      // Initialize the Hdlc Receiver
      result = OtxHdlcExReceiverInit(
                  hReceiver,   
                  OTX_HDLC_FFT_FLAGS,             
                  1,                              
                  OTX_HDLC_CRC16,                 
                  OTX_FALSE,                      
                  OTX_FALSE,                      
                  OTX_HDLC_TR_NA,                 
                  OTX_TRUE                        
        ); 

      result = OtxHdlcExSS7SetFilter(
                  hReceiver, 
                  OTX_HDLC_SS7_FILTER_FISU | OTX_HDLC_SS7_FILTER_SHORT_LSSU | OTX_HDLC_SS7_FILTER_LONG_LSSU); 

  @xref
    <f OtxHdlcExSS7GetFilter>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSS7SetFilter(
  IN HDLCX_HANDLE     hHdlcDevice,            // @parm Handle to the HDLC Pipe
  IN OTX_UINT32       filterMask              // @parm Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);


/*******************************************************************************

  @func
    Returns the currently active filter mask for filtering out FISUs, LSSUs 
    and/or MSUs

  @ex |
      
      OTX_HANDLE hHandle;
      OTX_UINT32 filterMask;

      hHandle = g_sqDspHdlcReceiver.hHandle;

      // toggle OTX_HDLC_SS7_FILTER_FISU_ALL filter mask on/off 
      result = OtxHdlcExSS7GetFilter(
                  hHandle, 
                  &filterMask); 
      
      if (filterMask & OTX_HDLC_SS7_FILTER_FISU_ALL) {
        filterMask &= ~OTX_HDLC_SS7_FILTER_FISU_ALL;
      } else {
        filterMask |= OTX_HDLC_SS7_FILTER_FISU_ALL;
      }

      // reset the filter mask
      result = OtxHdlcSS7SetFilter(
                  hHandle, 
                  filterMask); 

  @xref
    <f OtxHdlcExSS7SetFilter>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSS7GetFilter(
  IN HDLCX_HANDLE     hHdlcDevice,            // @parm Handle to the HDLC Device
  OUT OTX_UINT32      *pFilterMask            // @parm Currently active Filter Mask (see HDLC_SS7_FILTER_XXXX Macros)
);


/*******************************************************************************

  @func
    Send a message (LSSU, MSU, or arbitrary data) on a SS7 pipe. After the 
    sending of the message has been completed, continues sending the new FISU 
    provided as a parameter to the function.

  @ex |
  
    OtxHdlcSS7FisuS msu;
    OtxHdlcSS7FisuS *pFisu;
    OTX_UINT8 sqHdlcFrame[300];

    memcpy(&msu, pFisu, OTX_HDLC_SS7_CB_FISU);        // Build the first 3 bytes of the MSU
    msu.li = 10 - OTX_HDLC_SS7_CB_FISU;               // Set the Length Indicator
    memcpy(sqHdlcFrame, &msu, OTX_HDLC_SS7_CB_FISU);  // Put the first 3 bytes of the MSU in the HDLC frame buffer
    for (i = OTX_HDLC_SS7_CB_FISU; i < 10; i++) {     // Add the payload the HDLC frame buffer
      sqHdlcFrame[i + OTX_HDLC_SS7_CB_FISU] = (OTX_UINT8) i;
    }

    // Send MSU Frame with length of 10
    result = OtxHdlcExSS7SendData(
                g_sqDspHdlcSender.hHandle, 
                sqHdlcFrame, 
                10, 
                pFisu, 
                DEMO_TASK_REF, 
                OTX_TIME_INFINITY); 
    
    @xref
    <f OtxHdlcExSendData>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExSS7SendData(
  IN HDLCX_HANDLE     hHdlcDevice,            // @parm Handle to the Logical HDLC Device
  IN OTX_UINT8        *psqData,               // @parm Data (MSU, LSSU, or other arbitrary frame) to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data to be sent
  OtxHdlcSS7FisuS     *pFisu,                 // @parm Next FISU to be sent after the data
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Load an HDLC frame to be repeated for the HDLC pipe and begin transmission. 
    The Repeat Data is an HDLC frame which will be sent continuosly when there 
    is no other user data frames to be sent. User data can be sent with 
    <f OtxHdlcExSendData> function on which case the user data frame in 
    interleaved between two consecutive Repeat Data frames. After the sending 
    of user data frame has been completed, the HDLC controller will resume the 
    sending of the Repeat Data frame.

    This function can be used to continously send SS7 FISUs or LSSUs.

  @ex |

      OTX_UINT8 *pRepeatFrame = (OTX_UINT8 *)"\x10\x11\x12\x13\x14";
      OTX_HANDLE hHandle;

      hHandle = g_sqDspHdlcSender.hHandle;

      // start Repeat Data Frames with length of 5 from HDLC Sender 
      result = OtxHdlcExStartRepeatData(
                  hHandle, 
                  pRepeatFrame, 
                  5, 
                  DEMO_TASK_REF, 
                  OTX_TIME_INFINITY); 

  @xref
   <f OtxHdlcExStopRepeatData>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExStartRepeatData(
  IN HDLCX_HANDLE     hHdlcDevice,            // @parm Handle to the HDLC Device
  IN OTX_UINT8        *psqData,               // @parm Repeat Data frame to be sent
  IN OTX_UINT32       nDataLen,               // @parm Length of the data frame to be sent
  IN OTX_TASK_REF     nTaskRef,               // @parm See <t OTX_TASK_REF>
  IN OTX_TIME         nWaitMaxMs              // @parm See <t OTX_TIME>
);


/*******************************************************************************

  @func
    Stop sending of Repeat Data frames on a specified HDLC pipe.

  @ex |

    OTX_HANDLE hHandle;

    hHandle = g_sqDspHdlcSender.hHandle;

    // stop Repeat Data Frames from HDLC Sender 
    result = OtxHdlcExStopRepeatData(hHandle);

  @xref
   <f OtxHdlcExStartRepeatData>

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExStopRepeatData(
  IN HDLCX_HANDLE     hHdlcDevice             // @parm Handle to the HDLC Device
);


/*******************************************************************************
  @mfunc

    Retrieve internal FIFO statistics from an HDLC device

*******************************************************************************/
OTX_RESULT HDLCX_API OtxHdlcExGetFifoStats(
  IN  HDLCX_HANDLE    hHdlcDevice,         // @parm Handle to the HDLC Device
  OUT OTX_UINT32      *pcoFrameBuffer,     // @parm Maximum number of frames that the FIFO can contain for this device (if not limited by maximum number of bytes)
  OUT OTX_UINT32      *pcbOctetBuffer,     // @parm Maximum number of bytes that the FIFO can contain for this device (if not limited by the number of frames)
  OUT OTX_UINT32      *pcbOctetsInUse,     // @parm Number of bytes that the FIFO is currently using
  OUT OTX_UINT32      *pcoFramesInUse,     // @parm Number of frames that the FIFO currently contains
  OUT OTX_UINT32      *pcbOctetsPeakUse,   // @parm Maximum number of octets that the FIFO has ever used
  OUT OTX_UINT32      *pcoFramesPeakUse,   // @parm Maximum number of frames that the FIFO has ever contained
  OUT OTX_UINT32      *pcoFramesLost       // @parm Number of frames lost due to a full FIFO condition
);


#ifdef __cplusplus
  } // extern "C"
#endif

#endif

/*--------------------------------------------------------------------------*\

  Copyright (C) Odin TeleSystems Inc., 2004


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

\*--------------------------------------------------------------------------*/
