#ifndef OTX_T1E1_H
#define OTX_T1E1_H
/*******************************************************************************

  Copyright (C) 1998 - 2004 Odin TeleSystems Inc., All Rights Reserved

  @doc PublicHwApi OtxT1E1

  OTX Generic Hardware Driver API

  @module
  OtxT1E1 | This file contains function prototypes for the T1/E1 Line Interface devices

  @contents2
  OtxT1E1.h |

  @index func | OtxT1E1

  @contents2
  OTX T1/E1 Line Interface Device Functions - OtxT1XXX(), OtxE1XXX() |
  The user functions available for the T1/E1 Line Interface Transceiver Devices

  @head3 OtxT1E1XXX(), OtxT1XXX(), and OtxE1XXX() Function Reference |

  @index func | OtxT1E1

*******************************************************************************/

#include "OtxDef.h"

#include "OtxT1E1D.h"

#ifdef __cplusplus
  extern "C" {
#endif


/*******************************************************************************

  @func
    Sets up the T1/E1 Line Interface transceiver in either T1 or E1 mode.

  @ex |

    // set default configuration data of T1/E1.
    // NOTE that the default values should match to your application
    
    OtxT1ConfigOptionsS *pT1CfgData
    OtxE1ConfigOptionsS *pE1CfgData,

    // T1 configuration data
    pT1CfgData->eSignalingMode             = OTX_T1_SM_CCS;
    pT1CfgData->eTxLineCode                = OTX_T1_LC_B8ZS;
    pT1CfgData->eRxLineCode                = OTX_T1_LC_B8ZS;
    pT1CfgData->eFrameFormat               = OTX_T1_FF_ESF;
    pT1CfgData->bCRC6Enable                = OTX_TRUE;
    pT1CfgData->eTxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
    pT1CfgData->eRxRemoteAlarmFormat       = OTX_T1_RA_YELLOW_A;
    pT1CfgData->bAutoResync                = OTX_TRUE;
    pT1CfgData->bAutoCrcBitInversion       = OTX_TRUE;
    pT1CfgData->eLfaSensitivity            = OTX_T1_LFA_2_OUT_OF_4;
    pT1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
    pT1CfgData->nLosSensitivity            = 128;
    pT1CfgData->nLosRecovery               = 128;
    pT1CfgData->nLineLength                = 10;
    pT1CfgData->bHighTxPower               = OTX_FALSE;
    pT1CfgData->bReceiveEqualizer          = OTX_FALSE;
    pT1CfgData->bfClearChannels            = 0x0;

    // E1 configuration data 
    pE1CfgData->eTxLineCode                = OTX_E1_LC_HDB3;
    pE1CfgData->eRxLineCode                = OTX_E1_LC_HDB3;
    pE1CfgData->eTxFrameFormat             = OTX_E1_FF_CRC4_MULTIFRAME;
    pE1CfgData->eRxFrameFormat             = OTX_E1_FF_CRC4_MULTIFRAME;
    pE1CfgData->eAisDetection              = OTX_E1_AIS_ETS300233;
    pE1CfgData->bfSiBits                   = 0x3;
    pE1CfgData->bfSaBits                   = 0x1F;
    pE1CfgData->bExtendedHDB3errorDetect   = OTX_FALSE;
    pE1CfgData->bAutoRegainMultiframe      = OTX_FALSE;
    pE1CfgData->bAutoResync                = OTX_TRUE;
    pE1CfgData->bAutoCrcBitInversion       = OTX_FALSE;
    pE1CfgData->bAutoRemoteAlarm           = OTX_FALSE;
    pE1CfgData->nLosSensitivity            = 128;
    pE1CfgData->nLosRecovery               = 128;
    pE1CfgData->nLineLength                = 10;
    pE1CfgData->bHighTxPower               = OTX_FALSE;
    pE1CfgData->bReceiveEqualizer          = OTX_FALSE;

    // configure T1/E1 transceiver    
    for (i = 0; i < 2; i++) {

      // set termination for the T1/E1 Front End
      result = OtxT1E1SetLineTermination(
                  g_hPhysDevs.hLi[i], 
                  OTX_T1E1_TERM_100_OHM); 

      // configure Transceivers
      if (g_demoEnv.eT1E1Mode == OTX_T1E1_MODE_T1) {
        // T1 mode
        result = OtxT1SetConfiguration(
                    g_hPhysDevs.hLi[i], 
                    &(g_T1E1CfgData.t1CfgData));  
      } else {
        // E1 mode
        result = OtxE1SetConfiguration(
                    g_hPhysDevs.hLi[i], 
                    &(g_T1E1CfgData.e1CfgData));  
      }
      
      // set T1 or E1 mode
      result = OtxT1E1SetLiMode(
                  g_hPhysDevs.hLi[i], 
                  g_demoEnv.eT1E1Mode); 
    }

  @xref
    <t OtxT1E1LiModeE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1SetLiMode(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1LiModeE       eT1E1Mode         // @parm T1 or E1 mode
);


/*******************************************************************************

  @func
    Configures the T1 Line Interface transceiver.

  @ex |
    // see example on
    @ex <f OtxT1E1SetLiMode> |

  @xref
    <f OtxT1GetConfiguration>
    <nl><f OtxE1SetConfiguration>
    <nl><t OtxT1ConfigOptionsS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1SetConfiguration(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1ConfigOptionsS  *pT1ConfigOptions // @parm T1 Configuration Options
);


/*******************************************************************************

  @func
    Retrieves the current T1 Line Interface configuration data.

  @ex |

    OtxT1ConfigOptionsS m_sT1CfgData[2];

    for (i = 0; i < 2; i++) {
      
      // get handle of Line Interface 
      result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                i, 
                i, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[i]); 
  	  
      result = OtxT1GetConfiguration(
                g_hPhysDevs.hLi[i], 
                &(m_sT1CfgData[i])); 
    }

  @xref
    <f OtxT1SetConfiguration>
    <nl><f OtxE1SetConfiguration>
    <nl><t OtxT1ConfigOptionsS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1GetConfiguration(
  IN  OTX_HANDLE          hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxT1ConfigOptionsS *pT1ConfigOptions // @parm T1 Configuration Data
);


/*******************************************************************************

  @func
    Configures the E1 Line Interface transceiver.

  @ex |
    // see example on
    @ex <f OtxT1E1SetLiMode> |

  @xref
    <f OtxE1GetConfiguration>
    <nl><f OtxT1SetConfiguration>
    <nl><t OtxE1ConfigOptionsS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1SetConfiguration(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxE1ConfigOptionsS  *pE1ConfigOptions // @parm E1 Configuration Data
);


/*******************************************************************************

  @func
    Retrieves the current E1 Line Interface configuration data.

  @ex |

    OtxE1ConfigOptionsS m_sE1CfgData[2];

    for (i = 0; i < 2; i++) {
      
      // get handle of Line Interface 
      result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                i, 
                i, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[i]); 
  	  
      result = OtxE1GetConfiguration(
                g_hPhysDevs.hLi[i], 
                &(m_sE1CfgData[i])); 
    }

  @xref
    <f OtxE1SetConfiguration>
    <nl><f OtxT1SetConfiguration>
    <nl><t OtxE1ConfigOptionsS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1GetConfiguration(
  IN  OTX_HANDLE hLineDevice,               // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxE1ConfigOptionsS *pE1ConfigOptions // @parm E1 Configuration Data
);


/*******************************************************************************

  @func
    Initiates a sending of an alarm or a series of alarms on the T1/E1 physical 
    layer (depending on alarmType)

  @ex |

    // start/stop sending Remote (Yellow) alarm on current Li
          
    static OTX_BOOL g_bSendingRemoteAlarm = OTX_FALSE;

    if (g_bSendingRemoteAlarm) {
      // stop sending Remote (Yellow) Alarm on current Li 
      result = OtxT1E1AlarmOff(
                  g_hPhysDevs.hLi[nCurrentLi], 
                  OTX_T1E1_ALARM_RAI); 
    } else {
      // send Remote (Yellow) Alarm on current Li
      result = OtxT1E1AlarmOn(
                  g_hPhysDevs.hLi[nCurrentLi], 
                  OTX_T1E1_ALARM_RAI); 
    }
    
    // update status
    g_bSendingRemoteAlarm = !g_bSendingRemoteAlarm;

  @xref
    <t OtxT1E1AlarmTypeE>
    <nl><f OtxT1E1AlarmOff>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1AlarmOn(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1AlarmTypeE    eAlarmType        // @parm Alarm type to be sent to the Remote End
);


/*******************************************************************************

  @func
    Clears an earlier initiated alarm.

  @ex |
    see example on
    @ex <f OtxT1E1AlarmOn> |  

  @xref
    <t OtxT1E1AlarmTypeE>
    <nl><f OtxT1E1AlarmOn>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1AlarmOff(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1AlarmTypeE    eAlarmType        // @parm Alarm type to turned off
);


/*******************************************************************************

  @func
    Loops the Line Interface, i.e. received E1/T1 data will be transmitted back
    on the transmit pairs.

  @ex |

    for (i = 0; i < 2; i++) {
      
      // get handle of Line Interface 
      result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                i, 
                i, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[i]); 
  	  
      // set external loopback mode 
      result = OtxT1E1SetLineLoop(
                g_hPhysDevs.hLi[i], 
                OTX_T1E1_LINE_LOOP); 
    }

  @xref
    <t OtxT1E1LoopE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1SetLineLoop(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  OtxT1E1LoopE            eLoopType         // @parm Loop Type: Line loop or Remote loop
);


/*******************************************************************************

  @func
    Generate, Monitor or disable 2^15-1 or 2^20-1 PRBS 

  @ex |
      
    OTX_INT32 nCurrentLi = 0;

    // get handle of Line Interface 
    result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                i, 
                i, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[nCurrent]); 
  	  
    // enable 2^15-1 PRBS Monitor on Current Li     
    result = OtxT1E1SetPrbsConfig(
                g_hPhysDevs.hLi[nCurrentLi], 
                OTX_T1E1_PRBS_MONITOR_ON, 
                OTX_E1_PRBS_2_15);  

  @xref
    <t OtxPrbsModeE>
    <t OtxPrbsPatternE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1SetPrbsConfig(
  IN OTX_HANDLE           hDevice,          // @parm Handle to the T1/E1 Transceiver Device
  IN OtxPrbsModeE         ePrbsMode,        // @parm PRBS Monitor, Generate or Disable
  IN OtxPrbsPatternE      ePrbsPattern      // @parm '0' for 2^15-1; '1' for 2^20-1
);    


/*******************************************************************************

  @func
    Initiates a resynchronization procedure of the pulse frame and the
    CRC-multiframe starting directly after the old framing candidate.


  @ex | 

    // force resynchronization on Line 0 and 1
    for (nLi = 0; nLi < 2; nLi++) {
      
      // get handle of Line Interface 
      result = OtxDrvOpenPhysicalDevice(
                  g_hPhysDevs.hBoardController, 
                  OTX_DEVICE_LI_T1E1, 
                  nLi, 
                  nLi, 
                  g_hEventQueue, 
                  &g_hPhysDevs.hLi[nLi]); 
  	  
      result = OtxT1E1ForceResynch(g_hPhysDevs.hLi[nLi]); 
    }

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1ForceResynch(
  IN OTX_HANDLE           hLineDevice       // @parm Handle to the T1/E1 Transceiver Device
);


/*******************************************************************************

  @func
    Creates a single bit defect on the E1/T1 span.
    After calling this function, the selected defect is inserted into 
    the transmit data stream at the next possible position. The defect is
    not repeated.

  @ex |

    OTX_INT32 nCurrentLi = 0;

    // get handle of Line Interface 
    result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                i, 
                i, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[nCurrentLi]); 

    // insert a Bipolar violation defect on current Line
    result = OtxT1E1InsertSingleBitDefect(
                g_hPhysDevs.hLi[nCurrentLi], 
                OTX_T1E1_DEFECT_BPV); 

  @xref
    <t OtxT1E1DefectE>
      
*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1InsertSingleBitDefect(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1DefectE       eDefectType       // @parm Type bit defect to be inserted into the T1/E1 span
);


/*******************************************************************************

  @func
    Read the physical line status of the T1/E1 Line Interface. The line status is
    defined by two factors: Loss Of Signal and Framing Error. Both of these two
    factors have to be false for Line Status to be defined as "OK". If either of 
    the factors are false, then the Line Status is defined as "DOWN".
    
  @ex |
  
    OtxT1E1Layer1StatusE eLayer1Status;
    OTX_INT32 nLi;

    for (nLi = 0; nLi < 2; nLi++) {
      // get handle of Line Interface 
      result = OtxDrvOpenPhysicalDevice(
                  g_hPhysDevs.hBoardController, 
                  OTX_DEVICE_LI_T1E1, 
                  nLi, 
                  nLi, 
                  g_hEventQueue, 
                  &g_hPhysDevs.hLi[nLi]); 
        
      // get line status of Li
      result = OtxT1E1GetLayer1Status(
                  g_hPhysDevs.hLi[nLi], 
                  &eLayer1Status); 

      // display the status
      printf("%s\n", eLayer1Status == OTX_T1E1_LAYER1_OK ? "OK" : "DOWN");
    }

  @xref
    <t OtxT1E1Layer1StatusE>
      
*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1GetLayer1Status(
  IN OTX_HANDLE             hLineDevice,    // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxT1E1Layer1StatusE  *eLayer1Status  // @parm T1/E1 Physical Layer Line Status. Either <e OtxT1E1ResultCodesE.OTX_T1E1_RES_L1_DOWN> or <e OtxT1E1ResultCodesE.OTX_T1E1_RES_L1_OK>
);


/*******************************************************************************

  @func
    Poll the current physical line status of the T1/E1 Line Interface.
    
  @ex |
  
    OtxT1E1Layer1StatusExS sLayer1Status;
        
    // get line status of Li
    result = OtxT1E1GetLayer1StatusEx(
                g_hPhysDevs.hLi, 
                &sLayer1Status); 

    // display the status
    printf("Red Alarm        = %s\n", (sLayer1Status.bRedAlarm) ? "YES" : "NO");
    printf("Blue Alarm       = %s\n", (sLayer1Status.bBlueAlarm) ? "YES" : "NO");
    printf("Yellow Alarm     = %s\n", (sLayer1Status.bYellowAlarm) ? "YES" : "NO");
    printf("Frame Error      = %s\n", (sLayer1Status.bFrameError) ? "YES" : "NO");
    printf("MultiFrame Error = %s\n", (sLayer1Status.bMultiFrameError) ? "YES" : "NO");
    

  @xref
    <t OtxT1E1Layer1StatusExS>
      
*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1GetLayer1StatusEx(
  IN OTX_HANDLE             hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  OUT OtxT1E1Layer1StatusExS  *sLayer1Status  // @parm T1/E1 Physical Layer Line Status.
);


/*******************************************************************************

  @func
    Enable or Disable (tri-state) the Transmit pair of the Line Interface.
    Tristating the transmit pair will in effect disconnect the transmit pair 
    from the outgoing span.

  @ex |

    OTX_INT32 nCurrentLi = 0;

    // get handle of Line Interface 
    result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                nCurrentLi, 
                nCurrentLi, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[nCurrentLi]); 

    // tri-state Transmit Pins for Current Li
    result = OtxT1E1XmitPinControl(
                g_hPhysDevs.hLi[nCurrentLi], 
                OTX_FALSE);


*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1XmitPinControl(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OTX_BOOL             bTxPinEnable      // @parm OTX_TRUE enable, OTX_FALSE disables (tri-state) the Line Interface Transmit Pins
);


/*******************************************************************************

  @func
    Sets the line termination for the E1/T1 span of the Line Interface.

  @ex |
    // see example on
    @ex <f OtxT1E1SetLiMode> |

  @xref
    <t OtxT1E1TermModeE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1SetLineTermination(
  IN OTX_HANDLE           hLineDevice,      // @parm Handle to the T1/E1 Transceiver Device
  IN OtxT1E1TermModeE     eTermMode         // @parm Termination mode to match impedance of the cable connected to the Line Interface
);


/*******************************************************************************

  @func
    Increases the receiver sensitivity to detect resistively attenuated signals 
    of -20 dB. Typically used for in a monitor configuration.

  @comm
    This function is only valid for OTX_DEVICE_LI_T1E1 devices hosted by a 
    OTX_DEVICE_QUAD_CODEC device or a OTX_DEVICE_THOR_PCMCIA (Plus version) device.

  @ex |
  
    OTX_INT32 nCurrentLi = 0;

    // get handle of Line Interface 
    result = OtxDrvOpenPhysicalDevice(
                g_hPhysDevs.hBoardController, 
                OTX_DEVICE_LI_T1E1, 
                nCurrentLi, 
                nCurrentLi, 
                g_hEventQueue, 
                &g_hPhysDevs.hLi[nCurrentLi]); 

    // enable Monitor (-20dB) mode for Current Li
    result = OtxT1E1SetMonitorMode(
                g_hPhysDevs.hLi[nCurrentLi], 
                OTX_TRUE); 

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1E1SetMonitorMode(
  IN OTX_HANDLE           hLineDevice,        // @parm Handle to the T1/E1 Transceiver Device
  IN OTX_BOOL             bMonitorModeActive  // @parm If OTX_TRUE the receiver sensitivity is increased to detect resistively attenuated signals of -20 dB
);


/*******************************************************************************

  @func
    Set the FS/DL-Bit data for the <e OtxLogicalDeviceTypeE.OTX_LDEVICE_T1_DL_BIT_ACCESS>
    device.

  @comm
    Start FS/DL bit access by calling function <f OtxDrvEnable> and end it 
    with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode. 

  @ex |

    OTX_INT32 nCurrentLi = 0;

    // create DL-Bit Access logical devices hosted in the physical Line Interface devices
    for (i = 0; i < 2; i++) {
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_T1_DL_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1DlBitAccess[i]);
                  
      // Enable Logical DL-Bit Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1DlBitAccess[i],   
                  OTX_TASK_SYNC, 
                  5000L);
    }

    // send DL-Bit data: 0x09
    result = OtxT1SetDlBitData(
                g_hLogicDevs.hT1E1DlBitAccess[nCurrentLi], 
                0x09); 

  @xref
    <e OtxLogicalDeviceTypeE.OTX_LDEVICE_T1_DL_BIT_ACCESS>
    <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_SENT>
    <e OtxT1E1EventCodesE.OTX_T1E1_EC_DL_BITS_RECEIVED>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1SetDlBitData(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the T1 DL-Bit Access Logical Device
  IN OTX_UINT32           nDlBits           // @parm DL-Data to be transmitted.
                                            // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F12>:
                                            // <nl> <p nDlBits> should contain 8 DL bits.
                                            // <nl> Bit0 is transmitted in frame 2 and Bit7 in frame 12.
                                            // <nl> Remaining bits should be zero.
                                            // <nl> 
                                            // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_ESF>:                                           
                                            // <nl> <p nDlBits> should contain 16 DL bits.
                                            // <nl> Bit0 is transmitted in frame  1 and Bit7  in frame 15.
                                            // <nl> Bit8 is transmitted in frame 17 and Bit15 in frame 23.
                                            // <nl> Remaining bits should be zero.
                                            // <nl> 
                                            // <nl> Mode <e OtxT1FrameFormatE.OTX_T1_FF_F72>:
                                            // <nl> <p nDlBits> should contain 24 DL bits.
                                            // <nl> Bit0  is transmitted in frame 26 and Bit7  in frame 40
                                            // <nl> Bit8  is transmitted in frame 42 and Bit15 in frame 56.
                                            // <nl> Bit16 is transmitted in frame 58 and Bit23 in frame 72.
                                            // <nl> Remaining bits should be zero.

);


/*******************************************************************************

  @func
    Set the Bit Rob Signalling information (all channels).

  @comm
    Start bit-rob signalling by calling function <f OtxDrvEnable>
    and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode and  
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is the default)

  @ex |
    OTX_INT32 nCurrentLi = 0;
    OtxT1RobBitDataS  g_sqSendRobBitData[2];
    
    // create CAS logical devices hosted in the physical Line Interface devices
    for (i = 0; i < 2; i++) {
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_T1_ROB_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L); 
    }

    // zero transmitted Rob bits on current Li
    memset(&g_sqSendRobBitData[nCurrentLi], 0, sizeof(OtxT1RobBitDataS));
    result = OtxT1SetRobBitData(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                &g_sqSendRobBitData[nCurrentLi]); 

  @xref
    <f OtxT1GetBitRobData>
    <nl><t OtxT1BitRobDataS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1SetRobBitData(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the T1 BitRob Access Logical Device
  IN OtxT1RobBitDataS     *pRobBitData      // @parm Bit-robbed signalling data to be transmitted. See also <t OtxT1RobBitDataS>
);


/*******************************************************************************

  @func
    Get the current received bit rob signalling information (all channels).

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is the default)

  @ex |
    
    OTX_INT32 i;
    OTX_INT32 nCurrentLi = 0;
    OtxT1RobBitDataS  rcvRobBitData;
    
    // create CAS logical devices hosted in the physical Line Interface devices
    for (i = 0; i < 2; i++) {
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_T1_ROB_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L); 
    }

    // get Rob Bit data from current Line
    result = OtxT1GetRobBitData(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                &rcvRobBitData);

    // display the data
    printf("A = %.6X, B = %.6X, C = %.6X, D = %.6X\n", rcvRobBitData.chA, rcvRobBitData.chB, rcvRobBitData.chC, rcvRobBitData.chD);
    printf("\n");
    printf("            ABCD              ABCD\n");
    printf("            ====              ====\n");
    for (i = 0; i < 24; i++) {
      char *pszValue;
      pszValue[0] = '\0';
      
      printf("Channel %.2d: ", i+1);
  
      sprintf(pszValue + strlen(pszValue), "%d",  (pRobBitData->chA >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pRobBitData->chB >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pRobBitData->chC >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pRobBitData->chD >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      
      printf("  ");
      if (i % 2) {
        printf("\n");
      }
    }
    printf("\n");
  } 

  @xref
    <f OtxT1SetBitRobData>
    <t OtxT1BitRobDataS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1GetRobBitData(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the T1 BitRob Access Logical Device
  OUT OtxT1RobBitDataS    *pRobBitData      // @parm Bit-robbed signalling data received. See also <t OtxT1RobBitDataS>
);


/*******************************************************************************

  @func
    Set the Bit Rob Signalling information (ABCD bits) for a specific channel (1-24).

  @comm
    Start bit-rob signalling by calling function <f OtxDrvEnable>
    and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @ex |
    OTX_INT32 nCurrentLi = 0;
  
    for (i = 0; i < 2; i++) {
      //set Channelized ABCD bit access on Line Interface
      result = OtxDrvSetAttributeValueBool(
                  g_hPhysDevs.hLi[i], 
                  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS, 
                  OTX_TRUE); 

      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_T1_ROB_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L); 
    }

    // setting transmitted Rob Bit data (all channels) to 0 current Line
    for (i = 0; i < 24; i++) {
      result = OtxT1SetChannelizedRobBitData(
                  g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                  i+1, 
                  0); 
    }

  @xref
    <f OtxT1GetChannelizedBitRobData>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1SetChannelizedRobBitData(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the T1 S-Bit Access Logical Device
  IN OTX_INT32            nChannel,         // @parm Channel (1 - 24)
  IN OTX_UINT8            nABCDBits         // @parm The ABCD bits for this channel to be transmitted (A=bit3,B=bit2,C=bit1,D=bit0)
);


/*******************************************************************************

  @func
    Get the current received bit rob signalling (ABCD bits) for a specific 
    channel (1-24).

  @devnote
    Only meaningful in T1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE
  @ex |
    OTX_INT32 nCurrentLi = 0;
    OTX_UINT8 nABCDBits;
  
    for (i = 0; i < 2; i++) {
      //set Channelized ABCD bit access on Line Interface
      result = OtxDrvSetAttributeValueBool(
                  g_hPhysDevs.hLi[i], 
                  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS, 
                  OTX_TRUE); 

      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_T1_ROB_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L); 
    }

    // read received bitrob data on current Li
    for (i = 0; i < 24; i++) {
      result = OtxT1GetChannelizedRobBitData(
                  g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                  i+1, 
                  &nABCDBits);
      
      // display the data            
      printf("   Channel %.2d: 0x%X", i+1, nABCDBits);
      if (i % 2) {
        printf("\n");
      }
    }

  @xref
    <f OtxT1SetChannelizedBitRobData>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxT1GetChannelizedRobBitData(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the T1 Bit Rob Access Logical Device
  IN OTX_INT32            nChannel,         // @parm Channel (1 - 24)
  OUT OTX_UINT8           *pnABCDBits       // @parm The ABCD bits for this channel are returned in the lower nibble of this parameter (A=bit3,B=bit2,C=bit1,D=bit0)
);


/*******************************************************************************

  @func
    Return the value of the received SaX bits. Returns 8-bits received
    during the last CRC-Multiframe.

  @comm
    Start signalling by calling function <f OtxDrvEnable> and end it
    with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode

  @ex |
    
    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_S_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L);
    }
   
    // read service word data (Sa and Si bits) from all Lis
    for (nLi = 0; nLi < 2; nLi++) {
      OTX_UINT16 nSaPos;
      OTX_UINT8 nSaVal;
          
      for (nSaPos = 4; nSaPos <= 8; nSaPos++) {
        result =  OtxE1GetSaBit(
                     g_hLogicDevs.hT1E1Cas[nLi], 
                     (OtxE1SaBitE)(OTX_E1_SA4 + (nSaPos - 4)), 
                     &nSaVal);

        // display them
        printf("  SA%d=%.2X \n", nSaPos, nSaVal);
      }
    }

  @xref
    <f OtxE1SetSaBit>
    <nl><t OtxE1SaBitE>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1GetSaBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 S-Bit Access Logical Device
  IN  OtxE1SaBitE         eSaBit,           // @parm Sa bit to retrieve. See also <t OtxE1SaBitE>
  OUT OTX_UINT8           *pSaVal           // @parm Last 8 received Sa-bits
);


/*******************************************************************************

  @func
    Set the value to be sent at the SaX bits. Eight bits to be sent can be
    specified per Sa-bit.

  @comm
    In CRC-Multiframe format, one bit is sent in the corresponsing Sa-bit
    location of time-slot 0 in every other frame (in frames that do not
    contain frame alignment information). The least significant bit of the
    saVal byte is sent first in frame 1 of the multiframe and the most significant
    bit of the Byte is sent last in frame 15 of the multiframe.

    In Doubleframe format one bit of the saVal word is sent in the every other
    frame starting from the least significant bit.

    Start signalling by calling function <f OtxDrvEnable> and end it
    with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode

  @ex |
    
    OTX_INT32 nCurrentLi = 0;
    OTX_UINT8 nSaVal;
    char wValStr[12];

    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_S_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L);
    }

    // set Sa4 values 0x04 on current Line
    result = OtxE1SetSaBit(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                OTX_E1_SA4, 
                0x04); 

  @xref
    <f OtxE1SaBitsEnable>
    <nl><f OtxE1SaBitsDisable>
    <nl><f OtxE1GetSaBit>
    <nl><t OtxE1SaBitE>


*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1SetSaBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 S-Bit Access Logical Device
  IN OtxE1SaBitE          eSaBit,           // @parm Sa bit to set. See also <t OtxE1SaBitE>
  IN OTX_UINT8            nSaVal            // @parm Byte (8 bits) to be sent. LSB will be sent first.
);


/*******************************************************************************

  @func
    Retrieve the value to Si bits received in the last frame.

  @comm
    In Doubleframe format, these are the first bits of each frame.

    In CRC-Multiframe format, the Si bit are the first bits of frames 13 and
    15. In CRC-Multiframe format these bits are also known as the E-bits.

  @devnote
    Only meaningful in E1 mode

  @ex |
    
    OTX_UINT8 nSi1Val;
    OTX_UINT8 nSi2Val;

    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_S_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L);
    }

    for (nLi = 0; nLi < 2; nLi++) {
      result =  OtxE1GetSiBit(
                   g_hLogicDevs.hT1E1Cas[nLi], 
                   &nSi1Val, 
                   &nSi2Val);
      // display 
      printf("  Si1=%.1X", nSi1Val);
      printf("  Si2=%.1X \n", nSi2Val);
    }

  @xref
    <f OtxE1SetSiBit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1GetSiBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 S-Bit Access Logical Device
  OUT OTX_UINT8           *pSi1Val,         // @parm Value of the FAS Si-bit in doubleframe format
                                            // or Si (E) bit in frame 13 in CRC-multiframe format
  OUT OTX_UINT8           *pSi2Val          // @parm Value of the service word Si bit in DoubleFrame format
                                            // or Si (E) bit in frame 15 in CRC-multiframe format
);


/*******************************************************************************

  @func
    Set the value to be sent at the Si bit positions.

  @comm
    In Doubleframe format, these are the first bits of each frame.

    In CRC-Multiframe format, the Si bit are the first bits of frames 13 and
    15. In CRC-Multiframe format these bits are also known as the E-bits.

  @devnote
     Only meaningful in E1 mode

  @ex |

    OTX_INT32 nCurrentLi = 0;

    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_S_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 

      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 
                  5000L);
    }

    // set Si-bit values (both 0 on Si1 and Si2) on current Li
    result = OtxE1SetSiBit(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                0, 
                0); 

  @xref
    <f OtxE1GetSiBit>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1SetSiBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 S-Bit Access Logical Device
  IN OTX_UINT8            nSi1Val,          // @parm Value of the FAS Si-bit in doubleframe format
                                            // or Si (E) bit in frame 13 in CRC-multiframe format
  IN OTX_UINT8            nSi2Val           // @parm Value of the service word Si bit in DoubleFrame format
                                            // or Si (E) bit in frame 15 in CRC-multiframe format
);


/*******************************************************************************

  @func
    Set the E1 ABCD Signalling information for Channel Associated
    Signalling (CAS)

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is default)

  @ex |
    
    OTX_INT32 nCurrentLi = 0;
    OtxE1ABCDBitDataS  g_sqSendABCDBitData[2];

    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_ABCD_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 
                    
      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 5000L); 

    }

    // set ABCD-bits to ABCD = 0101 on all CAS channels on current Li
    g_sqSendABCDBitData[nCurrentLi].chA = 0x00000000;
    g_sqSendABCDBitData[nCurrentLi].chB = 0xffffffff;
    g_sqSendABCDBitData[nCurrentLi].chC = 0x00000000;
    g_sqSendABCDBitData[nCurrentLi].chD = 0xffffffff;
    
    result = OtxE1SetABCDBit(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                &g_sqSendABCDBitData[nCurrentLi]); 

  @xref
    <f OtxE1GetABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1SetABCDBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OtxE1ABCDBitDataS    *pABCDBitData     // @parm ABCD CAS signalling data to be transmitted. See also <t OtxE1ABCDBitDataS>
);


/*******************************************************************************

  @func
    Get the latest received E1 ABCD Signalling information for
    Channel Associated Signalling (CAS)

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_FALSE (which is default)

  @ex |

    OTX_INT32 i;
    OTX_INT32 nCurrentLi = 0;
    OtxE1ABCDBitDataS  rxABCDBitData;

    for (i = 0; i < 2; i++) {
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_ABCD_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 
                    
      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 5000L); 

    }
    
    // get ABCD bit data from current Line
    result = OtxE1GetABCDBit(
                g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                &rxABCDBitData); 

    // display the data
    printf("A = %.6X, B = %.6X, C = %.6X, D = %.6X\n", rxABCDBitData.chA, rxABCDBitData.chB, rxABCDBitData.chC, rxABCDBitData.chD);
    printf("\n");
    printf("            ABCD              ABCD\n");
    printf("            ====              ====\n");
    for (i = 1; i <= 30; i++) {
      char *pszValue;
      pszValue[0] = '\0';
   
      printf("Channel %.2d: ", i);

      sprintf(pszValue + strlen(pszValue), "%d",  (pABCDBitData->chA >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pABCDBitData->chB >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pABCDBitData->chC >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      sprintf(pszValue + strlen(pszValue), "%d",  (pABCDBitData->chD >> (nCasCh - 1)) & 0x01 ? 1 : 0);
      
      if (!(i % 2)) {
        printf("\n");
      }
    }
    printf("\n");

  @xref
    <f OtxE1SetABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1GetABCDBit(
  IN  OTX_HANDLE          hLogicDevice,     // @parm Handle to the E1 ABCD-Bit Access Logical Device
  OUT OtxE1ABCDBitDataS   *pABCDBitData     // @parm ABCD CAS signalling data received. See also <t OtxE1ABCDBitDataS>
);


/*******************************************************************************

  @func
    Set the E1 ABCD Signalling information for a specific channel (1-30)
    for Channel Associated Signalling (CAS).

  @comm
    Start signalling by calling function <f OtxDrvEnable> for the
    logical device and end it with function <f OtxDrvDisable>

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @ex |
    
    OTX_INT32 nCurrentLi = 0;

    for (i = 0; i < 2; i++) {
      //set Channelized ABCD bit access on Line Interface
      result = OtxDrvSetAttributeValueBool(
                  g_hPhysDevs.hLi[i], 
                  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS, 
                  OTX_TRUE); 
      
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_ABCD_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 
                    
      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 5000L); 

    }

    // set transmitted ABCD bit data (for all channels) to 0101 (0x05) on current Li
    for (i = 1; i <= 30; i++) {
      result = OtxE1SetChannelizedABCDBit(
                  g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                  i, 
                  0x05); 
    }

  @xref
    <f OtxE1GetChannelizedABCDBit>
    <nl><t OtxE1ABCDBitDataS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1SetChannelizedABCDBit(
  IN OTX_HANDLE           hLogicDevice,     // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OTX_INT32            nChannel,         // @parm CAS Channel (1 - 30)
  IN OTX_UINT8            nABCDBits         // @parm The ABCD bits for this channel to be transmitted (A=bit3,B=bit2,C=bit1,D=bit0)
);


/*******************************************************************************

  @func
    Get the latest received E1 ABCD Signalling information for a specific 
    channel (1-30) for Channel Associated Signalling (CAS).

  @devnote
    Only meaningful in E1 mode and
    if the <e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>
    is set to OTX_TRUE

  @ex |

    OTX_INT32 i;
    OTX_UINT8 nABCDBits;
    OTX_INT32 nCurrentLi = 0;

    for (i = 0; i < 2; i++) {
      //set Channelized ABCD bit access on Line Interface
      result = OtxDrvSetAttributeValueBool(
                  g_hPhysDevs.hLi[i], 
                  OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS, 
                  OTX_TRUE); 
      
      // create CAS logical devices hosted in the physical Line Interface devices
      result = OtxDrvCreateLogicalDevice( 
                  g_hPhysDevs.hLi[i], 
                  OTX_LDEVICE_E1_ABCD_BIT_ACCESS, 
                  g_hEventQueue, 
                  i, 
                  &g_hLogicDevs.hT1E1Cas[i]); 
                    
      // enable Logical T1/E1 Cas Devices
      result = OtxDrvEnable(
                  g_hLogicDevs.hT1E1Cas[i],   
                  OTX_TASK_SYNC, 5000L); 

    }

    for (i = 1; i <= 30; i++) {
      result = OtxE1GetChannelizedABCDBit(
                  g_hLogicDevs.hT1E1Cas[nCurrentLi], 
                  i, 
                  &nABCDBits); 
      
      // display the data
      printf("   Channel %.2d: 0x%X", i, nABCDBits);
      if (!(i % 2)) {
        printf("\n");
      }
    }

  @xref
    <f OtxE1SetChannelizedABCDBit>
    <nl><e OtxT1E1AttributesE.OTX_ATTR_T1E1_CHANNELIZED_ABCD_BIT_ACCESS>

*******************************************************************************/
OTX_EXPORT OTX_RESULT OTX_API OtxE1GetChannelizedABCDBit(
  IN  OTX_HANDLE          hLogicDevice,     // @parm Handle to the E1 ABCD-Bit Access Logical Device
  IN OTX_INT32            nChannel,         // @parm CAS Channel (1 - 30)
  OUT OTX_UINT8           *pnABCDBits       // @parm The ABCD bits for this channel are returned in the lower nibble of this parameter (A=bit3,B=bit2,C=bit1,D=bit0)
);



//***************************************************************************************
#ifdef _TURNSTONE_
//***************************************************************************************

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1SetActiveMode(
  IN OTX_HANDLE hDevice,          // Handle to the T1/E1 Transceiver Device
  IN TsT1E1ModeE eActiveMode);    // Mode Terminate/Monitor/Bridge/Loopback

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1StartPrbsPattern(IN OTX_HANDLE hDevice);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1StopPrbsPattern(IN OTX_HANDLE hDevice);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1SetPrbsConfig(IN OTX_HANDLE hDevice,
					IN TsT1E1IoInPrbsConfigOptionsS *pPrbsConfigOptions);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1SetPrbsMonitor(IN OTX_HANDLE hDevice,
								   IN OTX_UINT8 nMonitorPrbs);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1ChangeT1Config(IN OTX_HANDLE hDevice, IN ConfigTypeE configType);


#ifndef _WIN32_WCE
OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1StartSecTimer(IN OTX_HANDLE hDevice,IN OTX_HANDLE hEvent);
#else //_WIN32_WCE
OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1StartSecTimer(IN OTX_HANDLE hDevice);
#endif //_WIN32_WCE


OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1EnableLiAutoDetection(IN OTX_HANDLE hDevice,
										  OtxT1FrameFormatE  ePreferredFF);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1EnablePulseAutoDetection(IN OTX_HANDLE hDevice);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1ErrorInsertion(IN OTX_HANDLE hDevice,
								   IN TsT1E1ErrorTypeE ErrorType);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1SetLBO(IN OTX_HANDLE hDevice,
						   IN TsT1E1LineBuildOutE eBuildOut);


#ifdef _TURNSTONE_TEST_
OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1TestSetRegister(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT8 uSetValue,OTX_UINT8 uResetFalc);
OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1TestGetRegister(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT8 *uGetValue);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1TestResetCounters(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress);

OTX_EXPORT OTX_RESULT OTX_API OtxTsT1E1TestGetCounter(IN OTX_HANDLE hDevice,
								OTX_UINT32 uRegAddress,OTX_UINT32 *uRegCount);
#endif //_TURNSTONE_TEST_

//***************************************************************************************
#endif   //_TURNSTONE_
//***************************************************************************************


#ifdef __cplusplus
  } // extern "C"
#endif

/*******************************************************************************

  Copyright (C) Odin TeleSystems Inc., 1998 - 2004


  All rights reserved. No part of this computer program may be reproduced,
  stored in any retrieval system, or transmitted, in any form or by any
  means, electronic, mechanical, photocopying, recording, or otherwise,
  without the prior written permission of Odin TeleSystems Inc., or without
  a permission stipulated in the agreement/contract under which this computer
  program was supplied.

*******************************************************************************/
#endif   // OTX_T1E1_H
